/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include "common.h"
#include "version.h"
#include <xincs.h>

#include <FXJPGImage.h>
#include <FXPNGImage.h>
#include <FXPNGIcon.h>
#include <FXArray.h>
#include <fxkeys.h>

#include "version.h"
#include "GMAbout.h"
#include "GMWizard.h"
#include "GMWindow.h"
#include "GMTrackList.h"
#include "GMList.h"
#include "GMRemote.h"
#include "GMTag.h"
#include "GMThread.h"
#include "GMSearch.h"
#include "GMHeaderButton.h"
#include "GMSource.h"
#include "GMPlayerManager.h"
#include "GMDatabaseSource.h"
#include "GMTrackView.h"
#include "GMSourceView.h"
#include "GMAudioScrobbler.h"
#include "icons.h"
#include "GMIconTheme.h"

#include <xine.h>
#include "GMPlayer.h"

#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>

// Define Message Map
FXDEFMAP(GMWindow) GMWindowMap[]={
  //________Message_Type_____________________ID___________________________Message_Handler___
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_PLAY,              GMWindow::onUpdPlay),
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_PAUSE,             GMWindow::onUpdPause),
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_STOP,              GMWindow::onUpdStop),
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_NEXT,              GMWindow::onUpdNext),
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_PREV,              GMWindow::onUpdPrev),
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_REPEAT_ALL,       	GMWindow::onUpdRepeatAll),
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_REPEAT,    				GMWindow::onUpdRepeat),
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_REPEAT_AB,    		  GMWindow::onUpdRepeatAB),
  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_REPEAT_OFF,    		GMWindow::onUpdRepeatOff),


  FXMAPFUNC(SEL_UPDATE,         		GMWindow::ID_SHUFFLE,    				GMWindow::onUpdShuffle),
  FXMAPFUNC(SEL_UPDATE,							GMWindow::ID_SLEEP,					    GMWindow::onUpdSleepTimer),
  FXMAPFUNC(SEL_UPDATE,        		 	GMWindow::ID_SHOW_TRACK,        GMWindow::onUpdShowTrack),
  FXMAPFUNC(SEL_UPDATE,					    GMWindow::ID_VOLUME_BUTTON,     GMWindow::onUpdVolumeButton),
  FXMAPFUNC(SEL_UPDATE,        		  GMWindow::ID_SHOW_MINIPLAYER,   GMWindow::onUpdShowMiniPlayer),


  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_QUIT,							GMWindow::onCmdQuit),
  FXMAPFUNC(SEL_SIGNAL,							GMWindow::ID_QUIT,							GMWindow::onCmdQuit),
//  FXMAPFUNC(SEL_SIGNAL,							GMWindow::ID_CHILD,							GMWindow::onCmdChild),
//  FXMAPFUNC(SEL_CLOSE,							0,															GMWindow::onCmdCloseWindow),
//  FXMAPFUNC(SEL_COMMAND,						FXTopWindow::ID_CLOSE,					GMWindow::onCmdCloseWindow),
  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_IMPORT_DIRS,				GMWindow::onCmdImportDirs),
  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_IMPORT_FILES,		  GMWindow::onCmdImportFiles),


  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_OPEN,							GMWindow::onCmdOpen),

  FXMAPFUNC(SEL_COMMAND,         		GMWindow::ID_REPEAT_ALL,       	GMWindow::onCmdRepeatAll),
  FXMAPFUNC(SEL_COMMAND,         		GMWindow::ID_REPEAT_AB,    			GMWindow::onCmdRepeatAB),
  FXMAPFUNC(SEL_COMMAND,         		GMWindow::ID_REPEAT,    				GMWindow::onCmdRepeat),
  FXMAPFUNC(SEL_COMMAND,         		GMWindow::ID_REPEAT_OFF,    	  GMWindow::onCmdRepeatOff),

  FXMAPFUNC(SEL_COMMAND,         		GMWindow::ID_SHUFFLE,    				GMWindow::onCmdShuffle),



  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_ABOUT,             GMWindow::onCmdAbout),

  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_PLAY,              GMWindow::onCmdPlay),
  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_PAUSE,             GMWindow::onCmdPause),
  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_STOP,              GMWindow::onCmdStop),
  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_NEXT,              GMWindow::onCmdNext),
  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_PREV,              GMWindow::onCmdPrev),

#if FOXVERSION >= FXVERSION(1,7,11)
  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_SHOW_FULLSCREEN,		GMWindow::onCmdShowFullScreen),
#endif
  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_SHOW_MINIPLAYER,   GMWindow::onCmdShowMiniPlayer),

  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_SHOW_TRACK,        GMWindow::onCmdShowTrack),

  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_PREFERENCES,       GMWindow::onCmdPreferences),
  FXMAPFUNC(SEL_COMMAND,            GMWindow::ID_RESET_COLORS,      GMWindow::onCmdResetColors),

  FXMAPFUNC(SEL_COMMAND,        		GMWindow::ID_TIMESLIDER,        GMWindow::onCmdTimeSlider),

  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_VOLUME_SLIDER,			GMWindow::onCmdVolume),
  FXMAPFUNC(SEL_CHANGED,						GMWindow::ID_VOLUME_SLIDER,			GMWindow::onCmdVolume),
  FXMAPFUNC(SEL_MOUSEWHEEL,					GMWindow::ID_VOLUME_BUTTON,     GMWindow::onCmdVolumeButton),
  FXMAPFUNC(SEL_MOUSEWHEEL,					0,															GMWindow::onCmdVolumeButton),


  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_HOMEPAGE,					GMWindow::onCmdHomePage),
  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_REPORT_ISSUE,	  	GMWindow::onCmdReportIssue),

  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_JOIN_LASTFM,	  	GMWindow::onCmdJoinLastFM),
  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_JOIN_GMM_LASTFM,	  	GMWindow::onCmdJoinGMMLastFM),

  FXMAPFUNC(SEL_COMMAND,						GMWindow::ID_SLEEP,					GMWindow::onCmdSleepTimer),

  };


// Object implementation
FXIMPLEMENT(GMWindow,FXMainWindow,GMWindowMap,ARRAYNUMBER(GMWindowMap))


void setWindowCursor(FXWindow * window,FXCursor * cur) {
  window->setDefaultCursor(cur);
  window->setDragCursor(cur);
  FXWindow * child=window->getFirst();
  while(child) {
    child->setDefaultCursor(cur);
    child->setDragCursor(cur);
    child=child->getNext();
    }
  }

static FXbool launchBrowser(const FXString & url) {
  static const char * browsers[]={"xdg-open","firefox","konqueror","opera","netscape",NULL};

  FXString path = FXSystem::getExecPath();
  FXString exec;

  for (int i=0;browsers[i]!=NULL;i++){
    exec = FXPath::search(path,browsers[i]);
    if (!exec.empty()) break;
    }

  if (exec.empty()) return false;

  exec += " " + url;

  pid_t pid = fork();
  if (pid==-1){ /// Failure delivered to Parent Process
      return false;
      }
  else if (pid==0) { /// Child Process


      int i = sysconf(_SC_OPEN_MAX);
      while (--i >= 3) {
        close(i);
        }

      execlp("/bin/sh", "sh", "-c",exec.text(),(char *)0);
      exit(EXIT_FAILURE);
      }
  else { /// Parent Process
    return true;
      }
  return true;
  }

//----------------------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------------------
GMWindow::GMWindow(FXApp* a,FXObject*tgt,FXSelector msg) : FXMainWindow(a,"Goggles Music Manager",NULL,NULL,DECOR_ALL,5,5,700,580) {

  cover_small=NULL;

  icontheme = new GMIconTheme;
  icontheme->load();

  createFonts();

  setIcon(icontheme->icon_applogo);
  setMiniIcon(icontheme->icon_applogo_small);

#if APPLICATION_BETA > 0
  setTitle(GMStringFormat("Goggles Music Manager %d.%d.%d-beta%d",APPLICATION_MAJOR,APPLICATION_MINOR,APPLICATION_LEVEL,APPLICATION_BETA));
#endif

  /// Set myself as the target
  setTarget(tgt);
  setSelector(msg);

  /// Popup Volume Menu
  volumecontrol = new FXPopup(this,POPUP_VERTICAL|FRAME_RAISED|FRAME_THICK|POPUP_SHRINKWRAP);
  volumeslider = new FXSlider(volumecontrol,this,GMWindow::ID_VOLUME_SLIDER,LAYOUT_FIX_HEIGHT|LAYOUT_FIX_WIDTH|SLIDER_VERTICAL|SLIDER_TICKS_RIGHT|SLIDER_TICKS_LEFT|SLIDER_INSIDE_BAR,0,0,20,100);
  volumeslider->setTickDelta(10);
  volumeslider->setRange(0,100);
  volumeslider->setIncrement(10);

  // Make menu bar
  FXMenuBar * menubar=new FXMenuBar(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
  menubar->setDragCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
  statusbar = new FXStatusBar(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER,0,0,0,0,3,3,2,2,3);

  new FXSeparator(this,LAYOUT_FILL_X|SEPARATOR_GROOVE|LAYOUT_SIDE_TOP);
  controlstatusseparator = new FXSeparator(this,LAYOUT_FILL_X|SEPARATOR_GROOVE|LAYOUT_SIDE_BOTTOM);
  controlstatusseparator->hide();

  controlframe  = new FXHorizontalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X,0,0,0,0,3,3,3,3);
  playbutton    = new FXButton(controlframe,tr("Play\tStart Playback\tStart Playback"),icontheme->icon_play,this,ID_PLAY,BUTTON_TOOLBAR|FRAME_RAISED|ICON_ABOVE_TEXT);
  pausebutton   = new FXButton(controlframe,tr("Pause\tPause\tPause Playback"),icontheme->icon_pause,this,ID_PAUSE,BUTTON_TOOLBAR|FRAME_RAISED|ICON_ABOVE_TEXT);
  stopbutton    = new FXButton(controlframe,tr("Stop\tStop Playback\tStop Playback"),icontheme->icon_stop,this,ID_STOP,BUTTON_TOOLBAR|FRAME_RAISED|ICON_ABOVE_TEXT);
                  new FXVerticalSeparator(controlframe,LAYOUT_FILL_Y|SEPARATOR_GROOVE);
  prevbutton    = new FXButton(controlframe,tr("Previous\tPlay Previous Track\tPlay previous track."),icontheme->icon_prev,this,ID_PREV,BUTTON_TOOLBAR|FRAME_RAISED|ICON_ABOVE_TEXT);
  nextbutton    = new FXButton(controlframe,tr("Next\tPlay Next Track\tPlay next track."),icontheme->icon_next,this,ID_NEXT,BUTTON_TOOLBAR|FRAME_RAISED|ICON_ABOVE_TEXT);
                  new FXVerticalSeparator(controlframe,LAYOUT_FILL_Y|SEPARATOR_GROOVE);
  timeslider    = new FXSlider(controlframe,this,ID_TIMESLIDER,LAYOUT_FILL_X|LAYOUT_CENTER_Y);
  timeslider->setRange(0,65535);
  timeslider->setTickDelta(13107);
  timeslider->setIncrement(5000);
  timeslider->disable();
  timelabel  =new FX7Segment(controlframe,"--:--",SEVENSEGMENT_SHADOW|LAYOUT_CENTER_Y);
  timelabel->setCellWidth(10);
  timelabel->setCellHeight(15);

  new FXVerticalSeparator(controlframe,LAYOUT_FILL_Y|SEPARATOR_GROOVE);
  volumebutton = new FXMenuButton(controlframe,"\tAdjust Volume\tAdjust Volume",NULL,volumecontrol,MENUBUTTON_NOARROWS|MENUBUTTON_ATTACH_LEFT|MENUBUTTON_UP|MENUBUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_CENTER_Y);
  volumebutton->setTarget(this);
  volumebutton->setSelector(ID_VOLUME_BUTTON);
  controldragcorner = new FXDragCorner(controlframe);
  controlseparator  = new FXSeparator(this,LAYOUT_FILL_X|SEPARATOR_GROOVE|LAYOUT_SIDE_TOP);

  FXVerticalFrame * mainframe = new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);

  mainsplitter = new FXSplitter(mainframe,LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_HORIZONTAL|SPLITTER_TRACKING);
  FXVerticalFrame * sourceframe  = new FXVerticalFrame(mainsplitter,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0);
  coverframe = new FXImageFrame(sourceframe,NULL,LAYOUT_BOTTOM|JUSTIFY_CENTER_X|JUSTIFY_CENTER_Y|LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK,0,0,0,0,5,5,5,5);
  coverframe->setBackColor(getApp()->getBackColor());
  coverframe->hide();

  sourceview   = new GMSourceView(sourceframe);
/*  FXVerticalFrame * vframe = new FXVerticalFrame(mainsplitter,0, 0,0,0,0,0,0,0,0);
  FXHorizontalFrame * filterframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X, 0,0,0,0,0,0,0,0);
  new FXLabel(filterframe,"Filter",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
  new FXTextField(filterframe,20,NULL,0,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK);
*/
  trackview    = new GMTrackView(mainsplitter);

  // File Menu
  filemenu=new FXMenuPane(this);
  new FXMenuTitle(menubar,tr("&Music"),NULL,filemenu);
  new FXMenuCommand(filemenu,tr("Import File(s)" UTF8_ELLIPSIS "\t\tImport Music into Library"),icontheme->icon_importfile,this,GMWindow::ID_IMPORT_FILES);
  new FXMenuCommand(filemenu,tr("Import Folder" UTF8_ELLIPSIS "\tCtrl-O\tImport Music from folder into Library"),icontheme->icon_import,this,GMWindow::ID_IMPORT_DIRS);
  new FXMenuSeparator(filemenu);
  new FXMenuCommand(filemenu,tr("Open File" UTF8_ELLIPSIS "\t\tOpen Stream or File"),NULL,this,GMWindow::ID_OPEN);
  new FXMenuCommand(filemenu,tr("Export" UTF8_ELLIPSIS "\t\tExport Selected Source to File"),icontheme->icon_export,sourceview,GMSourceView::ID_EXPORT);
  new FXMenuSeparator(filemenu);
  new FXMenuCommand(filemenu,tr("New Playlist" UTF8_ELLIPSIS "\t\tCreate a new playlist"),icontheme->icon_playlist,sourceview,GMSourceView::ID_NEW_PLAYLIST);
  new FXMenuSeparator(filemenu);
  new FXMenuCommand(filemenu,tr("&Quit\tCtrl-Q\tQuit the application."),icontheme->icon_exit,this,GMWindow::ID_QUIT);
  setWindowCursor(filemenu,getApp()->getDefaultCursor(DEF_ARROW_CURSOR));

  // Help Menu
  editmenu=new FXMenuPane(this);
  new FXMenuTitle(menubar,tr("&Edit"),NULL,editmenu);
  new FXMenuCommand(editmenu,tr("&Copy\tCtrl-C\tCopy Selected Tracks"),icontheme->icon_copy,trackview,GMTrackView::ID_COPY,MENU_AUTOGRAY);
  new FXMenuCommand(editmenu,tr("&Cut\tCtrl-X\tCut Selected Tracks"),icontheme->icon_cut,trackview,GMTrackView::ID_CUT,MENU_AUTOGRAY);
  new FXMenuCommand(editmenu,tr("&Paste\tCtrl-V\tPaste Clipboard Selection"),icontheme->icon_paste,trackview,GMTrackView::ID_PASTE,MENU_AUTOGRAY);
  new FXMenuSeparator(editmenu);
  new FXMenuCommand(editmenu,tr("Find"UTF8_ELLIPSIS"\tCtrl-F\tShow search filter."),GMIconTheme::instance()->icon_find,trackview,GMTrackView::ID_TOGGLE_FILTER);
  new FXMenuSeparator(editmenu);
  new FXMenuCommand(editmenu,tr("Preferences" UTF8_ELLIPSIS ),icontheme->icon_settings,this,GMWindow::ID_PREFERENCES);
  setWindowCursor(editmenu,getApp()->getDefaultCursor(DEF_ARROW_CURSOR));

  viewmenu=new FXMenuPane(this);
  new FXMenuTitle(menubar,tr("&View"),NULL,viewmenu);
  new FXMenuCheck(viewmenu,tr("&Browse\tCtrl-B\tShow genre artist and album browser."),trackview,GMTrackView::ID_TOGGLE_BROWSER);
  new FXMenuCheck(viewmenu,tr("Show &Genres\tCtrl-G\tShow genre browser."),trackview,GMTrackView::ID_TOGGLE_GENRES);
  new FXMenuSeparator(viewmenu);
  new FXMenuCheck(viewmenu,tr("Show &Sources\tCtrl-S\tShow source browser "),sourceframe,FXWindow::ID_TOGGLESHOWN);
  new FXMenuSeparator(viewmenu);
#if FOXVERSION >= FXVERSION(1,7,11)
  fullscreencheck = new FXMenuCheck(viewmenu,tr("Show Full Screen\tF12\tToggle fullscreen mode."),this,ID_SHOW_FULLSCREEN);
#endif
  new FXMenuCheck(viewmenu,tr("Show Mini Player\tF11\tToggle Mini Player."),this,ID_SHOW_MINIPLAYER);

  new FXMenuSeparator(viewmenu);
  new FXMenuCommand(viewmenu,tr("&Configure Columns"UTF8_ELLIPSIS),NULL,trackview,GMTrackView::ID_CONFIGURE_COLUMNS);
//  new FXMenuCascade(viewmenu,tr("&Columns"),NULL,trackview->getColumnMenu());
  new FXMenuCascade(viewmenu,tr("&Sort"),icontheme->icon_sort,trackview->getSortMenu());
  new FXMenuSeparator(viewmenu);
  new FXMenuCommand(viewmenu,tr("&Jump to Current Track\tCtrl-J\tShow current playing track."),NULL,trackview,GMTrackView::ID_SHOW_CURRENT);
  setWindowCursor(viewmenu,getApp()->getDefaultCursor(DEF_ARROW_CURSOR));

  playmenu=new FXMenuPane(this);
  new FXMenuTitle(menubar,tr("&Control"),NULL,playmenu);
  new FXMenuCommand(playmenu,tr("Play\tCtrl-P\tStart playback."),icontheme->icon_play,this,ID_PLAY);
  new FXMenuCommand(playmenu,tr("Pause\t\tPause playback."),icontheme->icon_pause,this,ID_PAUSE);
  new FXMenuCommand(playmenu,tr("Stop\tCtrl-\\\tStop playback."),icontheme->icon_stop,this,ID_STOP);
  new FXMenuSeparator(playmenu);
  new FXMenuCommand(playmenu,tr("Previous Track\tCtrl-[\tPlay next track."),icontheme->icon_prev,this,ID_PREV);
  new FXMenuCommand(playmenu,tr("Next Track\tCtrl-]\tPlay previous track."),icontheme->icon_next,this,ID_NEXT);
  new FXMenuSeparator(playmenu);
  new FXMenuRadio(playmenu,tr("Repeat Off\tCtrl-,\tRepeat current track."),this,ID_REPEAT_OFF);
  new FXMenuRadio(playmenu,tr("Repeat Track\tCtrl-.\tRepeat current track."),this,ID_REPEAT);
  new FXMenuRadio(playmenu,tr("Repeat All Tracks\tCtrl-/\tRepeat all tracks."),this,ID_REPEAT_ALL);
  new FXMenuCheck(playmenu,tr("Repeat A-B\tCtrl-T\tRepeat section of track."),this,ID_REPEAT_AB);
  new FXMenuCheck(playmenu,tr("Shuffle Mode\tAlt-R\tPlay tracks in random order."),this,ID_SHUFFLE);
  new FXMenuSeparator(playmenu);
  new FXMenuCheck(playmenu,tr("Sleep Timer\t\tSetup sleeptimer."),this,ID_SLEEP);
  setWindowCursor(playmenu,getApp()->getDefaultCursor(DEF_ARROW_CURSOR));

  helpmenu=new FXMenuPane(this);
  new FXMenuTitle(menubar,tr("&Help"),NULL,helpmenu,LAYOUT_RIGHT);
  new FXMenuCommand(helpmenu,tr("&Homepage"),icontheme->icon_homepage,this,GMWindow::ID_HOMEPAGE);
  new FXMenuCommand(helpmenu,tr("&Report Issue" UTF8_ELLIPSIS),NULL,this,GMWindow::ID_REPORT_ISSUE);
  new FXMenuSeparator(helpmenu);
  new FXMenuCommand(helpmenu,tr("&Sign up for last.fm" UTF8_ELLIPSIS),NULL,this,GMWindow::ID_JOIN_LASTFM);
  new FXMenuCommand(helpmenu,tr("&Join GMM on last.fm" UTF8_ELLIPSIS"\t\tJoin the Goggles Music Manager group on last.fm"UTF8_ELLIPSIS),NULL,this,GMWindow::ID_JOIN_GMM_LASTFM);
  new FXMenuSeparator(helpmenu);
  new FXMenuCommand(helpmenu,tr("&About" UTF8_ELLIPSIS),icontheme->icon_info,this,GMWindow::ID_ABOUT);
  setWindowCursor(helpmenu,getApp()->getDefaultCursor(DEF_ARROW_CURSOR));

  getAccelTable()->addAccel(parseAccel("Ctrl-W"),this,FXSEL(SEL_CLOSE,0));
  getAccelTable()->addAccel(parseAccel("/"),trackview,FXSEL(SEL_COMMAND,GMTrackView::ID_TOGGLE_FILTER));
  }


void GMWindow::init() {
  sourceview->init();
  }

//----------------------------------------------------------------------------------
// Destructor
//----------------------------------------------------------------------------------
GMWindow::~GMWindow(){
  if (coverframe->getImage())
    delete coverframe->getImage();
  delete icontheme;
  }


//----------------------------------------------------------------------------------
// Create Fonts
//----------------------------------------------------------------------------------
void GMWindow::createFonts(){

#if FOXVERSION < FXVERSION(1,7,17)
  FXFontDesc fontdescription;
  getApp()->getNormalFont()->getFontDesc(fontdescription);
#else
  FXFontDesc fontdescription = getApp()->getNormalFont()->getFontDesc();
#endif
  fontdescription.weight = FXFont::Bold;
  font_thick = new FXFont(getApp(),fontdescription);
  font_thick->create();
/*
  /// Create a fixed font, about the same size as the normal font
  FXint size = getApp()->getNormalFont()->getSize();
  font_fixed = new FXFont(getApp(),"Courier",(int)size/10,FXFont::Normal,FXFont::Straight,FONTENCODING_DEFAULT,FXFont::NonExpanded,FXFont::Modern|FXFont::Fixed);
  font_fixed->create();
*/
  }





//----------------------------------------------------------------------------------
// Create
//----------------------------------------------------------------------------------
void GMWindow::create(){

  mainsplitter->setSplit(0,getApp()->reg().readIntEntry("window","source-list-split",200));

  configureStatusbar(GMPlayerManager::instance()->getPreferences().gui_show_status_bar);

  /// Set initial Toolbar Configuration
  configureToolbar(GMPlayerManager::instance()->getPreferences().gui_toolbar_docktop,
                   GMPlayerManager::instance()->getPreferences().gui_toolbar_showlabels,
                   GMPlayerManager::instance()->getPreferences().gui_toolbar_bigicons,
                   true);

  FXMainWindow::create();

  /// Initialize the window size & position
  if (getApp()->reg().readIntEntry("window","x",-1)!=-1) {
    FXint xx=getApp()->reg().readIntEntry("window","x",getX());
    FXint yy=getApp()->reg().readIntEntry("window","y",getY());
    FXint ww=getApp()->reg().readIntEntry("window","width",500);
    FXint hh=getApp()->reg().readIntEntry("window","height",550);
    position(xx,yy,ww,hh);
    }
  else {
    place(PLACEMENT_SCREEN);
    }


  }

#if FOXVERSION >= FXVERSION(1,7,11)
void GMWindow::setFullScreen(FXbool show){
  if (show) {
    if (isMaximized()) restore();
    getApp()->reg().writeIntEntry("window","x",getX());
    getApp()->reg().writeIntEntry("window","y",getY());
    getApp()->reg().writeIntEntry("window","width",getWidth());
    getApp()->reg().writeIntEntry("window","height",getHeight());
    fullScreen();
    statusbar->setCornerStyle(false);
    controldragcorner->hide();
    fullscreencheck->setCheck(true);
    }
  else {
    restore();
    if (!statusbar->shown() &&  GMPlayerManager::instance()->getPreferences().gui_toolbar_docktop==false)
      controldragcorner->show();
    statusbar->setCornerStyle(true);
    fullscreencheck->setCheck(false);
    }
  }
#endif


void GMWindow::hide() {
#if FOXVERSION >= FXVERSION(1,7,11)
  getApp()->reg().writeBoolEntry("window","fullscreen",isFullScreen());
#endif
  getApp()->reg().writeBoolEntry("window","maximized",isMaximized());
#if FOXVERSION >= FXVERSION(1,7,11)
  if (isFullScreen() || isMaximized() || isMinimized() )
#else
  if (isMaximized() || isMinimized() )
#endif
    restore();
  FXMainWindow::hide();
  }

void GMWindow::show(){
  FXMainWindow::show();

#if FOXVERSION >= FXVERSION(1,7,11)	
    if (getApp()->reg().readBoolEntry("window","fullscreen",false)){
      getApp()->reg().writeBoolEntry("window","fullscreen",false);
      getApp()->reg().writeBoolEntry("window","maximized",false);

     if (getApp()->reg().readIntEntry("window","x",-1)!=-1) {
       FXint xx=getApp()->reg().readIntEntry("window","x",getX());
       FXint yy=getApp()->reg().readIntEntry("window","y",getY());
       FXint ww=getApp()->reg().readIntEntry("window","width",500);
       FXint hh=getApp()->reg().readIntEntry("window","height",550);
       position(xx,yy,ww,hh);
       }
     else {
       place(PLACEMENT_SCREEN);
       }
      setFullScreen(true);
      }
    else
#endif 

  if (getApp()->reg().readBoolEntry("window","maximized",false)){
      getApp()->reg().writeBoolEntry("window","fullscreen",false);
      getApp()->reg().writeBoolEntry("window","maximized",false);
    maximize();	
    }		
  }



FXbool GMWindow::question(const FXString & title,const FXString & labeltext,const FXString & accept,const FXString & cancel){
  FXDialogBox dialog(this,title,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,400,0,0,0,0,0,0,0);
  create_dialog_header(&dialog,title,labeltext,NULL);
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_BOTTOM|LAYOUT_FILL_X,0,0,0,0);
  new FXButton(closebox,accept,NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20);
  new FXButton(closebox,cancel,NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20);
  if (dialog.execute()) {
    return TRUE;
    }
  return FALSE;
  }



void GMWindow::update_volume_display(FXint level) {
  if (level<=0)
    volumebutton->setIcon(icontheme->icon_audio_volume_muted);
  else if (level<=33)
    volumebutton->setIcon(icontheme->icon_audio_volume_low);
  else if (level<=66)
    volumebutton->setIcon(icontheme->icon_audio_volume_medium);
  else
    volumebutton->setIcon(icontheme->icon_audio_volume_high);

  volumeslider->setValue(level);
  }

void GMWindow::update_elapsed_time(FXint minutes,FXint seconds,FXint position,FXbool playing,FXbool seekable) {
  if (playing) {
    timelabel->setText(GMStringFormat("%.2d:%.2d",minutes,seconds));

    if (seekable) {
      if (!timeslider->grabbed()){
        timeslider->setValue(position);
        timeslider->enable();
        }
      }
    else {
      timeslider->disable();
      }
    }
  else {
    timelabel->setText("--:--");
    timeslider->disable();
    }
  }
















//----------------------------------------------------------------------------------
// Message Handlers
//----------------------------------------------------------------------------------
/*
long GMWindow::onCmdChild(FXObject*,FXSelector,void*){
  FXint pid;
  FXint status;
  while(1) {
    pid = waitpid(-1,&status,WNOHANG);
    if (pid>0) {
      continue;
      }
    break;
    }
  return 1;
  }
*/

long GMWindow::onCmdQuit(FXObject *,FXSelector,void*){

  sourceview->saveView();

  trackview->saveView();

  if (shown())
    GMPlayerManager::instance()->getPreferences().gui_last_startup=STARTUP_MAINWINDOW;
  else
    GMPlayerManager::instance()->getPreferences().gui_last_startup=STARTUP_REMOTE;

#if FOXVERSION >= FXVERSION(1,7,11)
  if (!isFullScreen() && !isMaximized() && !isMinimized() &&shown()) {
#else
  if (!isMaximized() && !isMinimized() && shown()) {
#endif
    getApp()->reg().writeIntEntry("window","x",getX());
    getApp()->reg().writeIntEntry("window","y",getY());
    getApp()->reg().writeIntEntry("window","width",getWidth());
    getApp()->reg().writeIntEntry("window","height",getHeight());
    }
#if FOXVERSION >= FXVERSION(1,7,11)
  getApp()->reg().writeBoolEntry("window","fullscreen",isFullScreen());
#endif
  getApp()->reg().writeBoolEntry("window","maximized",isMaximized());

  getApp()->reg().writeIntEntry("window","source-list-split",mainsplitter->getSplit(0));

  if (GMPlayerManager::instance()->getRemote())
    GMPlayerManager::instance()->getRemote()->writeRegistry();

  volumeslider->setTarget(NULL);
  volumeslider->setSelector(0);
  volumebutton->setMenu(NULL);
  delete volumecontrol;

  GMPlayerManager::instance()->exit();
  return 1;
  }



long GMWindow::onCmdAbout(FXObject *,FXSelector,void*){
  FXString version;
#if APPLICATION_BETA > 0
  version.format("%d.%d.%d-beta%d",APPLICATION_MAJOR,APPLICATION_MINOR,APPLICATION_LEVEL,APPLICATION_BETA);
#else
  version.format("%d.%d.%d",APPLICATION_MAJOR,APPLICATION_MINOR,APPLICATION_LEVEL);
#endif
  icontheme->loadAbout();
  FXAboutDialog dialog(this,"Goggles Music Manager",version,"Sander Jansen","2004-2008",icontheme->icon_about);
  dialog.execute(PLACEMENT_SCREEN);
  delete icontheme->icon_about;
  icontheme->icon_about=NULL;
  return 1;
  }


#if FOXVERSION >= FXVERSION(1,7,11)
long GMWindow::onCmdShowFullScreen(FXObject*,FXSelector,void*){
  if (isFullScreen()) {
    setFullScreen(false);
    }
  else {
    setFullScreen(true);
    }
  return 1;
  }
#endif


long GMWindow::onCmdShowMiniPlayer(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->showRemote();
  hide();
  return 1;
  }

long GMWindow::onUpdShowMiniPlayer(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->getRemote())
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_CHECK),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_UNCHECK),NULL);
  return 1;
  }


long GMWindow::onCmdShowTrack(FXObject *,FXSelector,void*){
  return 1;
  }

long GMWindow::onUpdShowTrack(FXObject *sender,FXSelector,void*){
  if (!GMPlayerManager::instance()->playing())
    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
  return 1;
  }

long GMWindow::onCmdResetColors(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->getPreferences().resetColors();
  return 1;
  }


struct ColorTheme {
  const FXchar* name;
  FXColor base;
  FXColor border;
  FXColor back;
  FXColor altback;
  FXColor fore;
  FXColor selback;
  FXColor selfore;
  FXColor tipback;
  FXColor tipfore;
  FXColor menuback;
  FXColor menufore;
  FXColor sourcefore;
  FXColor sourceback;
  FXColor playfore;
  FXColor playback;
  FXColor hilite;
  FXColor shadow;

  ColorTheme() {
    base      = FXApp::instance()->getBaseColor();
    border    = FXApp::instance()->getBorderColor();
    back      = FXApp::instance()->getBackColor();
    fore      = FXApp::instance()->getForeColor();
    selfore	  = FXApp::instance()->getSelforeColor();
    selback	  = FXApp::instance()->getSelbackColor();
    tipfore	  = FXApp::instance()->getTipforeColor();
    tipback	  = FXApp::instance()->getTipbackColor();
    menufore  = FXApp::instance()->getSelMenuTextColor();
    menuback  = FXApp::instance()->getSelMenuBackColor();
    shadow    = FXApp::instance()->getShadowColor();
    hilite    = FXApp::instance()->getHiliteColor();
    playfore  = GMPlayerManager::instance()->getPreferences().gui_playtext_color;
    playback  = GMPlayerManager::instance()->getPreferences().gui_play_color;
    sourcefore= GMPlayerManager::instance()->getPreferences().gui_sourceselecttext_color;
    sourceback= GMPlayerManager::instance()->getPreferences().gui_sourceselect_color;
    altback   = GMPlayerManager::instance()->getPreferences().gui_row_color;
    }
/*
  ColorTheme(const FXchar * _name,FXColor _base,FXColor _border,FXColor _back,FXColor _altback,FXColor _fore,FXColor _selback,FXColor _selfore,FXColor _menuback,FXColor _menufore,FXColor _tipback,FXColor _tipfore) :
    name(_name),
    base(_base),
    border(_border),
    back(_back),
    altback(_altback),
    fore(_fore),
    selback(_selback),
    selfore(_selfore),
    tipback(_tipback),
    tipfore(_tipfore),
    menuback(_menuback),
    menufore(_menufore),
    sourcefore(FXRGB(  0,  0,  0)),
    sourceback(FXRGB(210,230,210)),
    playfore(FXRGB(  0,  0,  0)),
    playback(FXRGB(210,230,210)),
	  hilite(makeHiliteColor(base)),
    shadow(makeShadowColor(base)) {
    }
*/

  ColorTheme(const FXchar * _name,FXColor _base,FXColor _border,FXColor _back,FXColor _altback,FXColor _fore,FXColor _selback,FXColor _selfore,FXColor _tipback,FXColor _tipfore,FXColor _psback=FXRGB(210,230,210),FXColor _psfore=FXRGB(  0,  0,  0)) :
    name(_name),
    base(_base),
    border(_border),
    back(_back),
    altback(_altback),
    fore(_fore),
    selback(_selback),
    selfore(_selfore),
    tipback(_tipback),
    tipfore(_tipfore),
    menuback(_selback),
    menufore(_selfore),
    sourcefore(_psfore),
    sourceback(_psback),
    playfore(_psfore),
    playback(_psback),
	  hilite(makeHiliteColor(base)),
    shadow(makeShadowColor(base)) {
    }


  void save() const {
    FXApp::instance()->setBaseColor(base);
    FXApp::instance()->setBorderColor(border);
    FXApp::instance()->setBackColor(back);
    FXApp::instance()->setForeColor(fore);
    FXApp::instance()->setSelbackColor(selback);
    FXApp::instance()->setSelforeColor(selfore);
    FXApp::instance()->setTipbackColor(tipback);
    FXApp::instance()->setTipforeColor(tipfore);
    FXApp::instance()->setSelMenuBackColor(menuback);
    FXApp::instance()->setSelMenuTextColor(menufore);
    FXApp::instance()->setShadowColor(shadow);
    FXApp::instance()->setHiliteColor(hilite);
    GMPlayerManager::instance()->getPreferences().gui_playtext_color=playfore;
    GMPlayerManager::instance()->getPreferences().gui_play_color=playback;
    GMPlayerManager::instance()->getPreferences().gui_sourceselecttext_color=sourcefore;
    GMPlayerManager::instance()->getPreferences().gui_sourceselect_color=sourceback;
    GMPlayerManager::instance()->getPreferences().gui_row_color=altback;
    }

  friend bool operator==(const ColorTheme& t1,const ColorTheme& t2);
  };

bool operator==(const ColorTheme& t1,const ColorTheme& t2) {
  return (t1.base==t2.base &&
      t1.border==t2.border &&
      t1.back==t2.back &&
      t1.altback==t2.altback &&
      t1.fore==t2.fore &&
      t1.selback==t2.selback &&
      t1.selfore==t2.selfore &&
      t1.tipback==t2.tipback &&
      t1.tipfore==t2.tipfore &&
      t1.menuback==t2.menuback &&
      t1.menufore==t2.menufore &&
      t1.sourcefore==t2.sourcefore &&
      t1.sourceback==t2.sourceback &&
      t1.playfore==t2.playfore &&
      t1.playback==t2.playback &&
      t1.hilite==t2.hilite &&
      t1.shadow==t2.shadow);
  }

const ColorTheme ColorThemes[]={
  ColorTheme("FOX", // name
             FXRGB(212,208,200), // base 
             FXRGB(  0,  0,  0), // boder
             FXRGB(255,255,255), // back
             FXRGB(240,240,240), // alt back
             FXRGB(  0,  0,  0), // fore
             FXRGB( 10, 36,106), // selback
             FXRGB(255,255,255), // selfore 
             FXRGB(255,255,225), // tipback
             FXRGB(0,0,0)), // tipfore 

  ColorTheme("Clearlooks"     , // name
             FXRGB(237,236,235), // base
             FXRGB(  0,  0,  0), // boder
             FXRGB(255,255,255), // back
             FXRGB(240,240,240), // alt back
             FXRGB( 26, 26, 25), // fore
             FXRGB(134,171,217), // selback
             FXRGB(255,255,255), // selfore 
             FXRGB(245,245,181), // tipback
             FXRGB(  0,  0,  0)),   // tipfore 

  ColorTheme("Honeycomb"       , // name
             FXRGB(213,215,209), // base
             FXRGB(  0,  0,  0), // boder
             FXRGB(255,255,255), // back
             FXRGB(238,238,238), // alt back
             FXRGB( 0, 0, 0), // fore
             FXRGB(227,167,  0), // selback
             FXRGB(255,255,255), // selfore 
             FXRGB(255,242,153), // tipback
             FXRGB( 64, 48, 0)), // tipfore 

  ColorTheme("Norway"          , // name
             FXRGB(235,226,210), // base
             FXRGB(  0,  0,  0), // boder
             FXRGB(253,252,251), // back
             FXRGB(238,238,238), // alt back
             FXRGB(  0,  0,  0), // fore
             FXRGB( 29,135,205), // selback
             FXRGB(255,255,255), // selfore 
             FXRGB(253,252,251), // tipback
             FXRGB(  0,  0,  0)),   // tipfore 

  ColorTheme("Oxygen"          , // name
             FXRGB(224,223,223), // base
             FXRGB(  0,  0,  0), // boder
             FXRGB(255,255,255), // back
             FXRGB(238,238,238), // alt back
             FXRGB( 20, 19, 18), // fore
             FXRGB( 65,141,212), // selback
             FXRGB(255,255,255), // selfore 
             FXRGB(192,218,255), // tipback
             FXRGB( 20, 19, 18)), // tipfore 

  ColorTheme("Obsidian Coast" ,  // name
             FXRGB( 48, 47, 47), // base
             FXRGB(  0,  0,  0), // boder
             FXRGB( 32, 31, 31), // back
             FXRGB( 38, 38, 38), // alt back
             FXRGB(224,223,220), // fore
             FXRGB(  24,72,128), // selback
             FXRGB(255,255,255), // selfore 
             FXRGB( 16, 48, 80), // tipback
             FXRGB(196,209,224),// tipfore 
             FXRGB(24,128,73), // psback   
             FXRGB(224,223,220)), // psfore   
 
  ColorTheme("Steel"       , // name
             FXRGB(224,223,216), // base
             FXRGB(  0,  0,  0), // boder
             FXRGB(255,255,255), // back
             FXRGB(238,238,238), // alt back
             FXRGB( 0, 0, 0), // fore
             FXRGB(123,161,173), // selback
             FXRGB(255,255,255), // selfore 
             FXRGB(220,231,235), // tipback
             FXRGB( 37, 34, 28)), // tipfore 

  ColorTheme("Wonton Soup"     , // name
             FXRGB (71, 76, 86), // base
             FXRGB(  0,  0,  0), // boder
             FXRGB( 58, 62, 70), // back
             FXRGB( 62, 66, 75), // alt back
             FXRGB(182,193,208), // fore
             FXRGB(117,133,153), // selback
             FXRGB(209,225,244), // selfore 
             FXRGB(182,193,208), // tipback
             FXRGB(42,44,48),		 // tipfore 
						 FXRGB(134,147,134), // psback   
						 FXRGB(209,225,244)),// psfore   

/*

  ColorTheme("Gnome"             ,FXRGB(214,215,214),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(  0,  0,128),FXRGB(255,255,255),FXRGB(  0,  0,128),FXRGB(255,255,255)),
  ColorTheme("Blue Slate"        ,FXRGB(239,239,239),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(103,141,178),FXRGB(255,255,255),FXRGB(103,141,178),FXRGB(255,255,255)),
  ColorTheme("CDE"	       			 ,FXRGB(156,153,156),FXRGB(  0,  0,  0),FXRGB(131,129,131),FXRGB(117,115,117),FXRGB(255,255,255),FXRGB( 49, 97,131),FXRGB(255,255,255),FXRGB( 49, 97,131),FXRGB(255,255,255)),
  ColorTheme("Digital CDE"       ,FXRGB( 74,121,131),FXRGB(  0,  0,  0),FXRGB( 55, 77, 78),FXRGB( 60, 84, 85),FXRGB(255,255,255),FXRGB( 82,102,115),FXRGB(255,255,255),FXRGB( 82,102,115),FXRGB(255,255,255)),
  ColorTheme("Desert FOX"        ,FXRGB(214,205,187),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(128,  0,  0),FXRGB(255,255,255),FXRGB(128,  0,  0),FXRGB(255,255,255)),
  ColorTheme("Galaxy"            ,FXRGB(239,239,239),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(103,141,178),FXRGB(255,255,255),FXRGB(103,141,178),FXRGB(255,255,255)),
  ColorTheme("Keramik"           ,FXRGB(234,233,232),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(169,209,255),FXRGB(  0,  0,  0),FXRGB(169,209,255),FXRGB(  0,  0,  0)),
  ColorTheme("Keramik Emerald"   ,FXRGB(238,238,230),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(134,204,134),FXRGB(  0,  0,  0),FXRGB(134,204,134),FXRGB(  0,  0,  0)),
  ColorTheme("Keramik White"     ,FXRGB(239,239,239),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(103,141,178),FXRGB(255,255,255),FXRGB(103,141,178),FXRGB(255,255,255)),
//  ColorTheme("Mandrake"          ,FXRGB(230,231,230),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB( 33, 68,156),FXRGB(255,255,255),FXRGB( 33, 68,156),FXRGB(255,255,255)),
  ColorTheme("Media Peach"       ,FXRGB(239,239,239),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(103,141,178),FXRGB(255,255,255),FXRGB(103,141,178),FXRGB(255,255,255)),
//  ColorTheme("Pale Gray"         ,FXRGB(214,214,214),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(  0,  0,  0),FXRGB(255,255,255)),
  ColorTheme("Oxygen"            ,FXRGB(222,221,220),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB( 63,126,189),FXRGB(255,255,255),FXRGB( 63,126,189),FXRGB(255,255,255)),
  ColorTheme("Norway"            ,FXRGB(235,226,210),FXRGB(  0,  0,  0),FXRGB(253,252,251),FXRGB( 62, 66, 75),FXRGB(  0,  0,  0),FXRGB(117,133,153),FXRGB(209,225,244),FXRGB(117,133,153),FXRGB(209,225,244),FXRGB(182,193,208),FXRGB(42,44,48)),

  ColorTheme("Plastik"           ,FXRGB(239,239,239),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(103,141,178),FXRGB(255,255,255),FXRGB(103,141,178),FXRGB(255,255,255)),
  ColorTheme("Redmond 95"        ,FXRGB(195,195,195),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(  0,  0,128),FXRGB(255,255,255),FXRGB(  0,  0,128),FXRGB(255,255,255)),
  ColorTheme("Redmond 2000"      ,FXRGB(212,208,200),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB(  0, 36,104),FXRGB(255,255,255),FXRGB(  0, 36,104),FXRGB(255,255,255)),
  ColorTheme("Redmond XP"        ,FXRGB(238,238,230),FXRGB(  0,  0,  0),FXRGB(255,255,255),FXRGB(240,240,240),FXRGB(  0,  0,  0),FXRGB( 74,121,205),FXRGB(255,255,255),FXRGB( 74,121,205),FXRGB(255,255,255)),
  ColorTheme("Solaris"           ,FXRGB(174,178,195),FXRGB(  0,  0,  0),FXRGB(147,151,165),FXRGB(132,135,148),FXRGB(  0,  0,  0),FXRGB(113,139,165),FXRGB(255,255,255),FXRGB(113,139,165),FXRGB(255,255,255)),
*/
  };


class GMPreferencesDialog : public FXDialogBox {
FXDECLARE(GMPreferencesDialog)
protected:
  FXIconPtr icon_lastfm;
protected:
  FXDataTarget target_alwaysremote;
  FXDataTarget target_startup;
  FXDataTarget target_closeishide;
  FXDataTarget target_filename_as_title;
  FXDataTarget target_directory_as_album;
  FXDataTarget target_directory_as_artist;
  FXDataTarget target_track_from_filelist;
  FXDataTarget target_replace_underscores;
  FXDataTarget target_user_title;
  FXDataTarget target_user_album;
  FXDataTarget target_user_artist;
  FXDataTarget target_user_category;
  FXDataTarget target_keywords;
  FXDataTarget target_format_template;
  FXDataTarget target_export_lowercase;
  FXDataTarget target_export_lowercase_extension;
  FXDataTarget target_export_underscore;
  FXDataTarget target_export_encoding;
  FXDataTarget target_close_audio;
  FXDataTarget target_pause_close_device;
  FXDataTarget target_gapless;
  FXDataTarget target_open_device_on_startup;
  FXDataTarget target_show_playing_albumcover;
  FXDataTarget target_show_albumcovers;
#ifdef HAVE_DBUS
  FXDataTarget target_dbus_notify_daemon;
  FXDataTarget target_dbus_tray_icon;
#endif
protected:
  FXString keywords;
public:
  FXTextField * user_title;
  FXTextField * user_album;
  FXTextField * user_artist;
  FXTextField * lastfm_username;
  FXTextField * lastfm_password;
  FXLabel     * lastfm_status;
	FXTextField * iconthemedir;
  FXCheckButton * check_audio_normalization;
  FXCheckButton * browser_icons;
  FXCheckButton * statusbarbutton;
  FXCheckButton * toolbar_bigicons;
  FXCheckButton * toolbar_showlabels;
  FXList        * colorpresets;
  FXListBox     * toolbar_docktop;
  FXbool password_set;
  FXFont * selectedfont;
  FXint theme;
  ColorTheme current; 
  ColorTheme selected;
public:
  enum {
    ID_TAG_CHANGED = FXDialogBox::ID_LAST,
    ID_LASTFM_USERNAME,
    ID_LASTFM_PASSWORD,
    ID_LASTFM_STATUS,
    ID_BASE_COLOR,
    ID_BACK_COLOR,
    ID_FORE_COLOR,
    ID_SHADOW_COLOR,
    ID_HILITE_COLOR,
    ID_SEL_BACK_COLOR,
    ID_SEL_FORE_COLOR,
    ID_MENU_BACK_COLOR,
    ID_MENU_FORE_COLOR,
    ID_TIP_BACK_COLOR,
    ID_TIP_FORE_COLOR,
    ID_SOURCE_BACK_COLOR,
    ID_SOURCE_FORE_COLOR,
    ID_PLAY_BACK_COLOR,
    ID_PLAY_FORE_COLOR,
    ID_ALTERNATIVE_BACK_COLOR,
    ID_BORDER_COLOR,
		ID_COLOR_THEME,
    ID_FONT,
    ID_CHANGE_FONT,
		ID_ICON_DIRECTORY
    };
public:
  long onCmdTagChanged(FXObject*,FXSelector,void*);
  long onCmdLastFMUserName(FXObject*,FXSelector,void*);
  long onCmdLastFMPassWord(FXObject*,FXSelector,void*);
  long onFocusLastFMPassWord(FXObject*,FXSelector,void*);
  long onCmdElementColor(FXObject*,FXSelector,void*);
  long onUpdElementColor(FXObject*,FXSelector,void*);
  long onCmdColorTheme(FXObject*,FXSelector,void*);
  long onUpdColorTheme(FXObject*,FXSelector,void*);
  long onUpdFont(FXObject*,FXSelector,void*);
  long onCmdChangeFont(FXObject*,FXSelector,void*);
  long onCmdAccept(FXObject*,FXSelector,void*);
#ifdef HAVE_EXTERNAL_ICONS
  long onCmdIconDirectory(FXObject*,FXSelector,void*);
#endif
protected:
  GMPreferencesDialog(){}
private:
  GMPreferencesDialog(const GMPreferencesDialog&);
  GMPreferencesDialog &operator=(const GMPreferencesDialog&);
public:
  GMPreferencesDialog(FXWindow * p);


  void initColorThemes() {
    for(FXuint i=0;i<ARRAYNUMBER(ColorThemes);i++){
      colorpresets->appendItem(ColorThemes[i].name,NULL,(void*)&ColorThemes[i]);
      }

    theme=-1;
    for(FXuint i=0;i<ARRAYNUMBER(ColorThemes);i++){
      if (current==ColorThemes[i]){
        theme=i;
        break;
        }
      }
      
    if (theme==-1){  
      colorpresets->prependItem("Current",NULL,(void*)&current);
      theme=0;
      }
    }


  void updateColorThemes() {
    theme=-1;

    for (FXint i=0;i<colorpresets->getNumItems();i++){
      if (selected==*(ColorTheme*)colorpresets->getItemData(i)){
        theme=i;
        return;
        }
      }
    
    if (theme==-1) {
      theme=colorpresets->getNumItems();
      colorpresets->appendItem("Custom",NULL,(void*)&selected);
      }
    else if (theme<colorpresets->getNumItems()-1) {
      if (colorpresets->getItemData(colorpresets->getNumItems()-1)==&selected)
        colorpresets->removeItem(colorpresets->getNumItems()-1);
      }
    }

  void updateColors();
  void redraw();

  ~GMPreferencesDialog();

  };

FXDEFMAP(GMPreferencesDialog) GMPreferencesDialogMap[]={
  FXMAPFUNC(SEL_COMMAND,GMPreferencesDialog::ID_TAG_CHANGED,GMPreferencesDialog::onCmdTagChanged),
  FXMAPFUNC(SEL_COMMAND,GMPreferencesDialog::ID_LASTFM_USERNAME,GMPreferencesDialog::onCmdLastFMUserName),
  FXMAPFUNC(SEL_CHANGED,GMPreferencesDialog::ID_LASTFM_USERNAME,GMPreferencesDialog::onCmdLastFMUserName),
  FXMAPFUNC(SEL_FOCUSIN,GMPreferencesDialog::ID_LASTFM_PASSWORD,GMPreferencesDialog::onFocusLastFMPassWord),
  FXMAPFUNC(SEL_COMMAND,GMPreferencesDialog::ID_LASTFM_PASSWORD,GMPreferencesDialog::onCmdLastFMPassWord),
  FXMAPFUNCS(SEL_COMMAND,GMPreferencesDialog::ID_BASE_COLOR,GMPreferencesDialog::ID_BORDER_COLOR,GMPreferencesDialog::onCmdElementColor),
  FXMAPFUNCS(SEL_UPDATE,GMPreferencesDialog::ID_BASE_COLOR,GMPreferencesDialog::ID_BORDER_COLOR,GMPreferencesDialog::onUpdElementColor),
  FXMAPFUNC(SEL_COMMAND,GMPreferencesDialog::ID_COLOR_THEME,GMPreferencesDialog::onCmdColorTheme),
  FXMAPFUNC(SEL_UPDATE,GMPreferencesDialog::ID_COLOR_THEME,GMPreferencesDialog::onUpdColorTheme),
  FXMAPFUNC(SEL_UPDATE,GMPreferencesDialog::ID_FONT,GMPreferencesDialog::onUpdFont),
  FXMAPFUNC(SEL_COMMAND,GMPreferencesDialog::ID_CHANGE_FONT,GMPreferencesDialog::onCmdChangeFont),
  FXMAPFUNC(SEL_COMMAND,FXDialogBox::ID_ACCEPT,GMPreferencesDialog::onCmdAccept)
#ifdef HAVE_EXTERNAL_ICONS
  ,FXMAPFUNC(SEL_COMMAND,GMPreferencesDialog::ID_ICON_DIRECTORY,GMPreferencesDialog::onCmdIconDirectory)
#endif

  };

FXIMPLEMENT(GMPreferencesDialog,FXDialogBox,GMPreferencesDialogMap,ARRAYNUMBER(GMPreferencesDialogMap))

GMPreferencesDialog::GMPreferencesDialog(FXWindow * p) : FXDialogBox(p,"Preferences",DECOR_BORDER|DECOR_TITLE,0,0,0,0,0,0,0,0,0,0), 
  password_set(false) {

  target_alwaysremote.connect(GMPlayerManager::instance()->getPreferences().gui_always_show_remote);
  target_startup.connect(GMPlayerManager::instance()->getPreferences().gui_startup);
  target_closeishide.connect(GMPlayerManager::instance()->getPreferences().gui_hide_player_when_close);

  target_filename_as_title.connect(GMPlayerManager::instance()->getPreferences().import_use_filename_for_title,this,ID_TAG_CHANGED);
  target_directory_as_album.connect(GMPlayerManager::instance()->getPreferences().import_use_directory_as_album,this,ID_TAG_CHANGED);
  target_directory_as_artist.connect(GMPlayerManager::instance()->getPreferences().import_use_directory_as_artist,this,ID_TAG_CHANGED);
  target_track_from_filelist.connect(GMPlayerManager::instance()->getPreferences().import_track_from_filelist);
  target_replace_underscores.connect(GMPlayerManager::instance()->getPreferences().import_replace_underscores);
  target_user_title.connect(GMPlayerManager::instance()->getPreferences().import_default_user_title);
  target_user_album.connect(GMPlayerManager::instance()->getPreferences().import_default_user_album);
  target_user_artist.connect(GMPlayerManager::instance()->getPreferences().import_default_user_artist);
  target_user_category.connect(GMPlayerManager::instance()->getPreferences().import_default_user_genre);
  target_keywords.connect(keywords);
  target_format_template.connect(GMPlayerManager::instance()->getPreferences().export_format_template);

  target_export_lowercase.connect(GMPlayerManager::instance()->getPreferences().export_lowercase);
  target_export_lowercase_extension.connect(GMPlayerManager::instance()->getPreferences().export_lowercase_extension);
  target_export_underscore.connect(GMPlayerManager::instance()->getPreferences().export_underscore);
  target_export_encoding.connect(GMPlayerManager::instance()->getPreferences().export_encoding);

  target_close_audio.connect(GMPlayerManager::instance()->getPreferences().play_close_stream);
  target_pause_close_device.connect(GMPlayerManager::instance()->getPreferences().play_pause_close_device);
  target_gapless.connect(GMPlayerManager::instance()->getPreferences().play_gapless);
  target_open_device_on_startup.connect(GMPlayerManager::instance()->getPreferences().play_open_device_on_startup);
  target_show_playing_albumcover.connect(GMPlayerManager::instance()->getPreferences().gui_show_playing_albumcover);
  target_show_albumcovers.connect(GMPlayerManager::instance()->getPreferences().gui_show_albumcovers);
#ifdef HAVE_DBUS
  target_dbus_notify_daemon.connect(GMPlayerManager::instance()->getPreferences().dbus_notify_daemon);
  target_dbus_tray_icon.connect(GMPlayerManager::instance()->getPreferences().dbus_tray_icon);
#endif


  GMPlayerManager::instance()->getPlayer()->checkInitialized();
  GMPlayerManager::instance()->getPreferences().getKeyWords(keywords);

  const FXuint labelstyle=LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT;
  const FXuint textfieldstyle=TEXTFIELD_ENTER_ONLY|LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN;

  FXGroupBox * grpbox;
  FXMatrix   * matrix;
  FXVerticalFrame * vframe;
  FXVerticalFrame * vframe2;
  

  icon_lastfm = new FXPNGIcon(getApp(),x20_lastfm_png);
  icon_lastfm->blend(getApp()->getBaseColor());


  FXVerticalFrame * main=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y);
  FXTabBook * tabbook=new FXTabBook(main,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT,0,0,0,0,0,0,0,0);


  new FXTabItem(tabbook,tr(" &General "),NULL);
  vframe = new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);

  grpbox =  new FXGroupBox(vframe,tr("Sort Options"),FRAME_GROOVE|LAYOUT_FILL_X);
  matrix = new FXMatrix(grpbox,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  new FXLabel(matrix,tr("Ignore leading words"),NULL,labelstyle);
  new FXTextField(matrix,10,&target_keywords,FXDataTarget::ID_VALUE,textfieldstyle);

  grpbox =  new FXGroupBox(vframe,tr("Album Covers"),FRAME_GROOVE|LAYOUT_FILL_X);
  new FXCheckButton(grpbox,tr("Show album cover of playing track\tShow album cover of playing track"),&target_show_playing_albumcover,FXDataTarget::ID_VALUE);
  new FXCheckButton(grpbox,tr("Show album covers in album browser\tShow album covers in album browser"),&target_show_albumcovers,FXDataTarget::ID_VALUE);

#ifdef HAVE_DBUS
	FXbool has_tray_icon=(!FXPath::search(FXSystem::getExecPath(),"gmm-tray").empty());
  if (GMPlayerManager::instance()->hasSessionBus()) {
    grpbox =  new FXGroupBox(vframe,tr("Tray Options"),FRAME_GROOVE|LAYOUT_FILL_X);
    if (has_tray_icon) new FXCheckButton(grpbox,tr("Show Tray Icon\tShow tray icon in the system tray."),&target_dbus_tray_icon,FXDataTarget::ID_VALUE); 
    new FXCheckButton(grpbox,tr("Show Track Change Notifications\tInform notification daemon of track changes."),&target_dbus_notify_daemon,FXDataTarget::ID_VALUE);
    }
#endif

  grpbox =  new FXGroupBox(vframe,tr("Last.fm"),FRAME_GROOVE|LAYOUT_FILL_X,0,0,0,0,3,3,0,0,0,0);
  vframe2 = new FXVerticalFrame(grpbox,LAYOUT_SIDE_RIGHT|PACK_UNIFORM_WIDTH,0,0,0,0,3,3,0,3,0,0);

  new FXLabel(vframe2,FXString::null,icon_lastfm,JUSTIFY_LEFT,0,0,0,0,8,0,0,0); 
  FXButton * linkbutton = new FXButton(vframe2,tr(" &Sign up for last.fm" UTF8_ELLIPSIS),NULL,GMPlayerManager::instance()->getMainWindow(),GMWindow::ID_JOIN_LASTFM,ICON_AFTER_TEXT|FRAME_RAISED|JUSTIFY_CENTER_Y|JUSTIFY_LEFT|BUTTON_TOOLBAR);
  linkbutton->setTextColor(FXRGB(0,0,255));
  linkbutton->setDefaultCursor(getApp()->getDefaultCursor(DEF_HAND_CURSOR));
  linkbutton = new FXButton(vframe2,tr(" &Join GMM on last.fm" UTF8_ELLIPSIS "\tJoin the Goggles Music Manager group on last.fm"UTF8_ELLIPSIS"\tJoin the Goggles Music Manager group on last.fm" UTF8_ELLIPSIS),NULL,GMPlayerManager::instance()->getMainWindow(),GMWindow::ID_JOIN_GMM_LASTFM,ICON_AFTER_TEXT|FRAME_RAISED|JUSTIFY_CENTER_Y|JUSTIFY_LEFT|BUTTON_TOOLBAR);
  linkbutton->setTextColor(FXRGB(0,0,255));
  linkbutton->setDefaultCursor(getApp()->getDefaultCursor(DEF_HAND_CURSOR));  
  if (GMPlayerManager::instance()->getAudioScrobbler()->isBanned()) {
    new FXLabel(grpbox,"This version of Goggles Music Manager is\n" 
                       "not supported by Last-FM. Please upgrade\n" 
                       "to a newer version of GMM.",NULL,JUSTIFY_LEFT);
    } 
  else {
    matrix = new FXMatrix(grpbox,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X,0,0,0,0);
    new FXLabel(matrix,tr("Username:"),NULL,labelstyle);
    lastfm_username = new FXTextField(matrix,20,this,ID_LASTFM_USERNAME,FRAME_SUNKEN|FRAME_THICK);
    new FXLabel(matrix,tr("Password:"),NULL,labelstyle);
    lastfm_password = new FXTextField(matrix,20,this,ID_LASTFM_PASSWORD,FRAME_SUNKEN|FRAME_THICK|TEXTFIELD_PASSWD);

    lastfm_username->setText(GMPlayerManager::instance()->getAudioScrobbler()->getUsername());
    if (GMPlayerManager::instance()->getAudioScrobbler()->hasPassword())
      lastfm_password->setText("1234567890");
    }


  new FXTabItem(tabbook,tr(" &Window "),NULL);
  vframe = new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);

  grpbox =  new FXGroupBox(vframe,tr("Window"),FRAME_GROOVE|LAYOUT_FILL_X);
  matrix = new FXMatrix(grpbox,2,MATRIX_BY_COLUMNS);
  new FXLabel(matrix,tr("Start Up:"),NULL,labelstyle);
  FXListBox * list_start = new FXListBox(matrix,&target_startup,FXDataTarget::ID_VALUE,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
  list_start->appendItem("Show Main Window");
  list_start->appendItem("Show Mini Remote");
  list_start->appendItem("Previous View");
  list_start->setNumVisible(3);

  new FXLabel(matrix,tr("Options:"),NULL,labelstyle);
  new FXCheckButton(matrix,tr("Always Show Remote"),&target_alwaysremote,FXDataTarget::ID_VALUE);
  new FXFrame(matrix,FRAME_NONE);
  new FXCheckButton(matrix,tr("Close button hides Main Window"),&target_closeishide,FXDataTarget::ID_VALUE);

  new FXFrame(matrix,FRAME_NONE);
  statusbarbutton = new FXCheckButton(matrix,tr("Show Status Bar"),NULL,0);
  statusbarbutton->setCheck(GMPlayerManager::instance()->getPreferences().gui_show_status_bar);

  new FXFrame(matrix,FRAME_NONE);
  browser_icons = new FXCheckButton(matrix,tr("Show Icons in Track Browser"),NULL,0);
  browser_icons->setCheck(GMPlayerManager::instance()->getPreferences().gui_show_browser_icons);


  grpbox =  new FXGroupBox(vframe,tr("Player Controls"),FRAME_GROOVE|LAYOUT_FILL_X);
  matrix = new FXMatrix(grpbox,2,MATRIX_BY_COLUMNS);
  new FXLabel(matrix,tr("Location:"),NULL,labelstyle);
  toolbar_docktop = new FXListBox(matrix,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X);
  toolbar_docktop->appendItem("Top");
  toolbar_docktop->appendItem("Bottom");
  toolbar_docktop->setNumVisible(2);

  if (GMPlayerManager::instance()->getPreferences().gui_toolbar_docktop)
    toolbar_docktop->setCurrentItem(0);
  else
    toolbar_docktop->setCurrentItem(1);

  new FXLabel(matrix,tr("Style:"),NULL,labelstyle);
  toolbar_showlabels = new FXCheckButton(matrix,tr("Show Labels"),NULL,0);
  toolbar_showlabels->setCheck(GMPlayerManager::instance()->getPreferences().gui_toolbar_showlabels);
  new FXFrame(matrix,FRAME_NONE);
  toolbar_bigicons = new FXCheckButton(matrix,tr("Large Icons"),NULL,0);
  toolbar_bigicons->setCheck(GMPlayerManager::instance()->getPreferences().gui_toolbar_bigicons);


  new FXTabItem(tabbook,tr(" &Appearance "),NULL);
  vframe = new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
  grpbox =  new FXGroupBox(vframe,tr("Colors"),FRAME_GROOVE|LAYOUT_FILL_X,0,0,0,0);

  matrix = new FXMatrix(grpbox,6,MATRIX_BY_COLUMNS|LAYOUT_SIDE_LEFT,0,0,0,0,0,0,0,0,0,0);

  new FXFrame(matrix,FRAME_NONE);  
  new FXLabel(matrix,"fg\tForeground Color",NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_CENTER_X);
  new FXLabel(matrix,"bg\tBackground Color",NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_CENTER_X);
  new FXLabel(matrix,"alt bg\tAlternative Background Color",NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_CENTER_X);
  new FXFrame(matrix,FRAME_NONE);
  new FXFrame(matrix,FRAME_NONE);

  new FXFrame(matrix,FRAME_NONE);
  new FXSeparator(matrix,SEPARATOR_LINE|LAYOUT_FILL_X);
  new FXSeparator(matrix,SEPARATOR_LINE|LAYOUT_FILL_X);
  new FXSeparator(matrix,SEPARATOR_LINE|LAYOUT_FILL_X);
  new FXFrame(matrix,FRAME_NONE);
  new FXFrame(matrix,FRAME_NONE);

  new FXLabel(matrix,tr("Normal\tNormal Text Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_FORE_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXColorWell(matrix,0,this,ID_BACK_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXColorWell(matrix,0,this,ID_ALTERNATIVE_BACK_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXLabel(matrix,tr("Base\tBase Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_BASE_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);

  new FXLabel(matrix,tr("Selected\tSelected Text Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_SEL_FORE_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXColorWell(matrix,0,this,ID_SEL_BACK_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXFrame(matrix,FRAME_NONE);
  new FXLabel(matrix,tr("Border\tBorder Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_BORDER_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);

  new FXLabel(matrix,tr("Menu\tMenu Text Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_MENU_FORE_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXColorWell(matrix,0,this,ID_MENU_BACK_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXFrame(matrix,FRAME_NONE);
  new FXLabel(matrix,tr("Hilite\tHilite Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_HILITE_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);

  new FXLabel(matrix,tr("Tooltip\tTooltip Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_TIP_FORE_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXColorWell(matrix,0,this,ID_TIP_BACK_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXFrame(matrix,FRAME_NONE);
  new FXLabel(matrix,tr("Shadow\tShadow Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_SHADOW_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);


  new FXLabel(matrix,tr("Source\tActive Source Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_SOURCE_FORE_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXColorWell(matrix,0,this,ID_SOURCE_BACK_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXFrame(matrix,FRAME_NONE);
  new FXFrame(matrix,FRAME_NONE);
  new FXFrame(matrix,FRAME_NONE);

  new FXLabel(matrix,tr("Playing\tPlaying Track Color"),NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT);
  new FXColorWell(matrix,0,this,ID_PLAY_FORE_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXColorWell(matrix,0,this,ID_PLAY_BACK_COLOR,COLORWELL_OPAQUEONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_ROW|LAYOUT_CENTER_Y,0,0,40,24);
  new FXFrame(matrix,FRAME_NONE);
  new FXFrame(matrix,FRAME_NONE);
  new FXFrame(matrix,FRAME_NONE);

  new FXSeparator(grpbox,SEPARATOR_GROOVE|LAYOUT_FILL_Y|LAYOUT_SIDE_LEFT);

  vframe2 = new FXVerticalFrame(grpbox,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  new FXLabel(vframe2,"Presets:");
  FXVerticalFrame * sunken = new FXVerticalFrame(vframe2,LAYOUT_SIDE_RIGHT|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0);
  colorpresets = new FXList(sunken,this,ID_COLOR_THEME,LIST_BROWSESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
	colorpresets->setNumVisible(9);

  initColorThemes();

  grpbox =  new FXGroupBox(vframe,tr("Font"),FRAME_GROOVE|LAYOUT_FILL_X,0,0,0,0);
  new FXButton(grpbox," Change"UTF8_ELLIPSIS" ",NULL,this,ID_CHANGE_FONT,BUTTON_NORMAL|LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y);
  new FXTextField(grpbox,20,this,ID_FONT,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_FILL_X|TEXTFIELD_NORMAL|TEXTFIELD_READONLY);

#ifdef HAVE_EXTERNAL_ICONS
  grpbox =  new FXGroupBox(vframe,tr("Icons"),FRAME_GROOVE|LAYOUT_FILL_X,0,0,0,0);
  new FXButton(grpbox,UTF8_ELLIPSIS,NULL,this,ID_ICON_DIRECTORY,BUTTON_NORMAL|LAYOUT_SIDE_RIGHT|LAYOUT_CENTER_Y);
  new FXLabel(grpbox,"Theme Directory:",NULL,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_SIDE_LEFT);
  iconthemedir=new FXTextField(grpbox,20,NULL,0,LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_FILL_X|TEXTFIELD_NORMAL|TEXTFIELD_READONLY);
	iconthemedir->setText(FXApp::instance()->reg().readStringEntry("user-icons","icon-base-dir",DEFAULT_ICON_DIRECTORY));
#endif

  new FXTabItem(tabbook,tr(" &Audio "),NULL);
  vframe = new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
  grpbox =  new FXGroupBox(vframe,tr("Options"),FRAME_GROOVE|LAYOUT_FILL_X);
  new FXCheckButton(grpbox,tr("Gapless playback"),&target_gapless,FXDataTarget::ID_VALUE);
  new FXCheckButton(grpbox,tr("Close audio device on pause."),&target_pause_close_device,FXDataTarget::ID_VALUE);
  new FXCheckButton(grpbox,tr("Turn off playback engine on stop."),&target_close_audio,FXDataTarget::ID_VALUE);
  new FXCheckButton(grpbox,tr("Turn on playback engine on startup.\tFor faster startup, playback engine is normally started when first track is played.\tFor faster startup, playback engine is normally started when first track is played."),&target_open_device_on_startup,FXDataTarget::ID_VALUE);

  grpbox =  new FXGroupBox(vframe,tr("Audio Plugins"),FRAME_GROOVE|LAYOUT_FILL_X);
  check_audio_normalization = new FXCheckButton(grpbox,tr("Volume Normalization"));

  if (GMPlayerManager::instance()->getPlayer()->hasVolumeNormalization() && GMPlayerManager::instance()->getPlayer()->getVolumeNormalization())
    check_audio_normalization->setCheck(true);

  new FXTabItem(tabbook,tr(" &Import "),NULL);
  vframe = new FXVerticalFrame(tabbook,FRAME_THICK|FRAME_RAISED);
  grpbox =  new FXGroupBox(vframe,tr("Incomplete Tags"),FRAME_GROOVE|LAYOUT_FILL_X);
  new FXCheckButton(grpbox,tr("Use filename for song title"),&target_filename_as_title,FXDataTarget::ID_VALUE);
  new FXCheckButton(grpbox,tr("Use directory name for album title"),&target_directory_as_album,FXDataTarget::ID_VALUE);
  new FXCheckButton(grpbox,tr("Use directory name for artist name"),&target_directory_as_artist,FXDataTarget::ID_VALUE);
  new FXCheckButton(grpbox,tr("Use scan order for track number"),&target_track_from_filelist,FXDataTarget::ID_VALUE);
  new FXSeparator(grpbox,LAYOUT_FILL_X|SEPARATOR_GROOVE);
  new FXCheckButton(grpbox,tr("Replace underscores with spaces"),&target_replace_underscores,FXDataTarget::ID_VALUE);

  grpbox =  new FXGroupBox(vframe,tr("Default Tags"),FRAME_GROOVE|LAYOUT_FILL_X);
  matrix = new FXMatrix(grpbox,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X);
  new FXLabel(matrix,tr("Title:"),NULL,labelstyle);
  user_title = new FXTextField(matrix,10,&target_user_title,FXDataTarget::ID_VALUE,textfieldstyle);
  new FXLabel(matrix,tr("Album:"),NULL,labelstyle);
  user_album = new FXTextField(matrix,10,&target_user_album,FXDataTarget::ID_VALUE,textfieldstyle);
  new FXLabel(matrix,tr("Artist:"),NULL,labelstyle);
  user_artist = new FXTextField(matrix,10,&target_user_artist,FXDataTarget::ID_VALUE,textfieldstyle);
  new FXLabel(matrix,tr("Genre:"),NULL,labelstyle);
  new FXTextField(matrix,10,&target_user_category,FXDataTarget::ID_VALUE,textfieldstyle);

  FXHorizontalFrame *closebox=new FXHorizontalFrame(main,LAYOUT_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
  new FXButton(closebox,tr(" &Close "),NULL,this,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20);

  onCmdTagChanged(NULL,0,NULL);
  }

GMPreferencesDialog::~GMPreferencesDialog(){
  }


long GMPreferencesDialog::onCmdAccept(FXObject*,FXSelector,void*) {
  getApp()->stopModal(this,TRUE);
  hide();

  GMWindow * mainwindow = GMPlayerManager::instance()->getMainWindow();
 
  if (!GMPlayerManager::instance()->getAudioScrobbler()->isBanned() && password_set) {  
    GMPlayerManager::instance()->getAudioScrobbler()->login(lastfm_username->getText(),lastfm_password->getText());
    }

  GMPlayerManager::instance()->getPreferences().setKeyWords(keywords);

  if (check_audio_normalization->getCheck()) {
    GMPlayerManager::instance()->getPlayer()->setVolumeNormalization(true);
    }
  else {
    GMPlayerManager::instance()->getPlayer()->setVolumeNormalization(false);
    }
  GMPlayerManager::instance()->getPlayer()->setupGapless();

  if (browser_icons->getCheck()!=GMPlayerManager::instance()->getPreferences().gui_show_browser_icons){
    GMPlayerManager::instance()->getPreferences().gui_show_browser_icons=browser_icons->getCheck();
    mainwindow->getTrackView()->updateIcons();
    }

  if (!(selected==current)) {
    GMIconTheme::instance()->load();
    GMIconTheme::instance()->updateToolbar(GMPlayerManager::instance()->getPreferences().gui_toolbar_bigicons);
    redraw();
    }

  mainwindow->configureToolbar((toolbar_docktop->getCurrentItem()==0),toolbar_showlabels->getCheck(),toolbar_bigicons->getCheck());
  mainwindow->configureStatusbar(statusbarbutton->getCheck());

  if (GMPlayerManager::instance()->getPreferences().gui_always_show_remote)
    GMPlayerManager::instance()->showRemote();
  else
    GMPlayerManager::instance()->hideRemote();
 
  if (GMPlayerManager::instance()->getPreferences().gui_show_playing_albumcover) {
    if (GMPlayerManager::instance()->playing())
      GMPlayerManager::instance()->update_track_display(false);
    }
  else {
    mainwindow->loadCover(FXString::null);
    }

  GMPlayerManager::instance()->update_tray_icon();

  // Key Words may have changed.
  mainwindow->getTrackView()->resort();
  mainwindow->getSourceView()->resort();
  return 1;
  }







#ifdef HAVE_EXTERNAL_ICONS
long GMPreferencesDialog::onCmdIconDirectory(FXObject*,FXSelector,void*){
  FXString  newdir = FXDirDialog::getOpenDirectory(this,"Select Theme Directory",FXApp::instance()->reg().readStringEntry("user-icons","icon-base-dir",DEFAULT_ICON_DIRECTORY));
	if (!newdir.empty()){
		FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-dir",newdir.text());
		GMIconTheme::instance()->load();
		GMIconTheme::instance()->updateToolbar(GMPlayerManager::instance()->getPreferences().gui_toolbar_bigicons);
		iconthemedir->setText(FXApp::instance()->reg().readStringEntry("user-icons","icon-base-dir",DEFAULT_ICON_DIRECTORY));
		}
	return 1;
	}
#endif





long GMPreferencesDialog::onCmdLastFMPassWord(FXObject*,FXSelector,void*){
//  fxmessage("onCmdLastFMPassWord %d\n",password_set);
  if (!GMPlayerManager::instance()->getAudioScrobbler()->isBanned() && password_set) {  
    GMPlayerManager::instance()->getAudioScrobbler()->login(lastfm_username->getText(),lastfm_password->getText());
    }
  return 1;
  }

long GMPreferencesDialog::onFocusLastFMPassWord(FXObject*,FXSelector,void*){
//  fxmessage("onFocusLastFMPassWord %d\n",password_set);
  if (password_set==false) {
    password_set=true;
    lastfm_password->setText(FXString::null,false);  
    }
  return 0;
  }

long GMPreferencesDialog::onCmdLastFMUserName(FXObject*,FXSelector sel,void*){
 // if (GMPlayerManager::instance()->getAudioScrobbler()->getUsername()!=lastfm_username->getText()){
    lastfm_password->setText(FXString::null,false);
    password_set=true;
    if (FXSELTYPE(sel)==SEL_COMMAND) lastfm_password->setFocus();
 //   }
  return 1;
  }

long GMPreferencesDialog::onCmdTagChanged(FXObject*,FXSelector,void*) {
  if (GMPlayerManager::instance()->getPreferences().import_use_filename_for_title)
    user_title->disable();
  else
    user_title->enable();

  if (GMPlayerManager::instance()->getPreferences().import_use_directory_as_album)
    user_album->disable();
  else
    user_album->enable();

  if (GMPlayerManager::instance()->getPreferences().import_use_directory_as_artist)
    user_artist->disable();
  else
    user_artist->enable();
  return 1;
  }

long GMPreferencesDialog::onCmdElementColor(FXObject*,FXSelector sel,void* rgba) {
  FXColor color = (FXColor)(FXuval)rgba;

  switch(FXSELID(sel)){
    case ID_BASE_COLOR : selected.base = color; 
                         selected.shadow = makeShadowColor(selected.base);
                         selected.hilite = makeHiliteColor(selected.base);
                         break;
   case ID_BORDER_COLOR : selected.border = color; break; 
   case ID_BACK_COLOR   : selected.back = color; break;
    case ID_FORE_COLOR  : selected.fore = color; break;
    case ID_SHADOW_COLOR : selected.shadow=color; break;
    case ID_HILITE_COLOR : selected.hilite=color; break;


    case ID_SEL_BACK_COLOR: selected.selback=color; break;
    case ID_SEL_FORE_COLOR: selected.selfore=color; break;
    case ID_MENU_BACK_COLOR: selected.menuback=color; break;
    case ID_MENU_FORE_COLOR: selected.menufore=color; break;
    case ID_TIP_BACK_COLOR: selected.tipback=color; break;
    case ID_TIP_FORE_COLOR: selected.tipfore=color; break;
    case ID_PLAY_BACK_COLOR: selected.playback=color; break;
    case ID_PLAY_FORE_COLOR: selected.playfore=color; break;
    case ID_SOURCE_BACK_COLOR: selected.sourceback=color; break;
    case ID_SOURCE_FORE_COLOR: selected.sourcefore=color; break;
    case ID_ALTERNATIVE_BACK_COLOR: selected.altback=color; break;

     }
  updateColors();

  updateColorThemes();
  return 1;
  }

long GMPreferencesDialog::onUpdElementColor(FXObject*sender,FXSelector sel,void*) {
  FXuval rgba = 0;
  switch(FXSELID(sel)){
    case ID_BASE_COLOR : rgba=selected.base; break;
    case ID_BORDER_COLOR : rgba=selected.border; break;
    case ID_BACK_COLOR : rgba=selected.back; break;
    case ID_FORE_COLOR : rgba=selected.fore; break;
    case ID_SHADOW_COLOR : rgba=selected.shadow; break;
    case ID_HILITE_COLOR : rgba=selected.hilite; break;

    case ID_SEL_BACK_COLOR: rgba=selected.selback; break;
    case ID_SEL_FORE_COLOR: rgba=selected.selfore; break;
    case ID_MENU_BACK_COLOR: rgba=selected.menuback; break;
    case ID_MENU_FORE_COLOR: rgba=selected.menufore; break;
    case ID_TIP_BACK_COLOR: rgba=selected.tipback; break;
    case ID_TIP_FORE_COLOR: rgba=selected.tipfore; break;
    case ID_PLAY_BACK_COLOR: rgba=selected.playback; break;
    case ID_PLAY_FORE_COLOR: rgba=selected.playfore; break;

    case ID_SOURCE_BACK_COLOR: rgba=selected.sourceback; break;
    case ID_SOURCE_FORE_COLOR: rgba=selected.sourcefore; break;
    case ID_ALTERNATIVE_BACK_COLOR: rgba=selected.altback; break;


    }
  sender->tryHandle(this,FXSEL(SEL_COMMAND,FXColorWell::ID_SETINTVALUE),&rgba);
  return 1;
  }
long GMPreferencesDialog::onCmdColorTheme(FXObject*,FXSelector,void*ptr) {
  theme=(FXint)(FXival)ptr;
  ColorTheme *theme_selected=reinterpret_cast<ColorTheme*>(colorpresets->getItemData(theme));

  selected.base = theme_selected->base;
  selected.border = theme_selected->border;
  selected.back = theme_selected->back;
  selected.altback = theme_selected->altback; 
  selected.fore = theme_selected->fore;
  selected.selfore	= theme_selected->selfore;
  selected.selback	= theme_selected->selback;
  selected.tipfore	= theme_selected->tipfore;
  selected.tipback	= theme_selected->tipback;
  selected.menufore= theme_selected->menufore;
  selected.menuback= theme_selected->menuback;
	selected.shadow = makeShadowColor(selected.base);
	selected.hilite   = makeHiliteColor(selected.base);
  selected.playback = theme_selected->playback;
  selected.playfore = theme_selected->playfore;
  selected.sourceback = theme_selected->sourceback;
  selected.sourcefore = theme_selected->sourcefore;

	updateColors();
	return 1;
	}

long GMPreferencesDialog::onUpdColorTheme(FXObject*sender,FXSelector,void*) {
  sender->tryHandle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETINTVALUE),&theme);
  return 1;
  }



void GMPreferencesDialog::updateColors(){  
  register FXWindow *w=FXApp::instance()->getRootWindow();

  FX7Segment * sevensegment;
  FXTextField * textfield;
  FXIconList * iconlist;
  FXList * list;
  FXListBox * listbox;
  FXTreeList * treelist;
  FXComboBox * combobox;
  FXButton * button;
  FXFrame * frame;
  FXLabel * label;
  FXPopup * popup;
  FXMenuTitle * menutitle;
  FXMenuCheck * menucheck;
  FXMenuRadio * menuradio;
  FXMenuCaption * menucaption;
  FXMenuSeparator * menuseparator;
  FXText * text;
  FXFoldingList * foldinglist;
  FXMDIChild * mdichild;
  FXTable * table;
  FXDockTitle * docktitle;
  FXPacker * packer;
  FXHeader * header;
  FXGroupBox * groupbox;
  FXScrollBar * scrollbar;
  FXSlider * slider;
  FXStatusLine * statusline;
  FXDragCorner * dragcorner;
  GMTreeList * gmtreelist;
  GMTrackList * gmtracklist;
  GMList * gmlist;
  FXRadioButton * radiobutton;
  FXCheckButton * checkbutton;
  FXToolTip * tooltip;
	FXImageFrame * imageframe;

  while(w){

    w->setBackColor(selected.base);

    if ((frame=dynamic_cast<FXFrame*>(w))) {

      frame->setBaseColor(selected.base);
      frame->setBackColor(selected.base);
      frame->setShadowColor(selected.shadow);
      frame->setHiliteColor(selected.hilite);
      frame->setBorderColor(selected.border);

      if ((label=dynamic_cast<FXLabel*>(w))) {
        label->setTextColor(selected.fore);
        if ((button=dynamic_cast<FXButton*>(w))) {
          if (dynamic_cast<FXListBox*>(button->getParent())){
            w->setBackColor(selected.back);
            }
          else {
            w->setBackColor(selected.base);
            }
          }
        else if ((checkbutton=dynamic_cast<FXCheckButton*>(w))) {
          checkbutton->setCheckColor(selected.fore);
          checkbutton->setBoxColor(selected.back);
          } 
        else if ((radiobutton=dynamic_cast<FXRadioButton*>(w))) {
          radiobutton->setRadioColor(selected.fore);
          radiobutton->setDiskColor(selected.back);
          } 
        }
      else if ((textfield=dynamic_cast<FXTextField*>(w))) {
        w->setBackColor(selected.back);
        textfield->setTextColor(selected.fore);
        textfield->setSelTextColor(selected.selfore);
        textfield->setSelBackColor(selected.selback);
        }
      else if ((docktitle=dynamic_cast<FXDockTitle*>(w))) {
        docktitle->setCaptionColor(selected.selfore);
        docktitle->setBackColor(selected.selback);
        }
      else if ((header=dynamic_cast<FXHeader*>(w))) {
        header->setTextColor(selected.fore);
        }
      else if ((statusline=dynamic_cast<FXStatusLine*>(w))) {
        statusline->setTextColor(selected.fore);
        }
      else if ((sevensegment=dynamic_cast<FX7Segment*>(w))) {
        sevensegment->setTextColor(selected.fore);
        }
      else if ((slider=dynamic_cast<FXSlider*>(w))) {
        slider->setSlotColor(selected.back);
        }
     else if ((imageframe=dynamic_cast<FXImageFrame*>(w))) { 
        imageframe->setBackColor(selected.back); /// fixme, only for coverframe in mainwindow
				}
     }
    else if ((packer=dynamic_cast<FXPacker*>(w))) {
      packer->setBaseColor(selected.base);
      packer->setBackColor(selected.base);
      packer->setShadowColor(selected.shadow);
      packer->setHiliteColor(selected.hilite);
      packer->setBorderColor(selected.border);
      if ((combobox=dynamic_cast<FXComboBox*>(w))) {
        w->setBackColor(selected.back);
        }
      else if ((listbox=dynamic_cast<FXListBox*>(w))) {
        w->setBackColor(selected.back);
        }
      else if ((groupbox=dynamic_cast<FXGroupBox*>(w))) {
        groupbox->setTextColor(selected.fore);
        }
      }
    else if ((popup=dynamic_cast<FXPopup*>(w))){
      popup->setBaseColor(selected.base);
      popup->setShadowColor(selected.shadow);
      popup->setHiliteColor(selected.hilite);
      popup->setBorderColor(selected.border);
      }
    else if ((menucaption=dynamic_cast<FXMenuCaption*>(w))) {
      w->setBackColor(selected.base);
      menucaption->setTextColor(selected.fore);
      menucaption->setSelTextColor(selected.menufore);
      menucaption->setSelBackColor(selected.menuback);
      menucaption->setShadowColor(selected.shadow);
      menucaption->setHiliteColor(selected.hilite);

      if ((menucheck=dynamic_cast<FXMenuCheck*>(w))) {
        menucheck->setBoxColor(selected.back);
        }
      else if ((menuradio=dynamic_cast<FXMenuRadio*>(w))) {
        menuradio->setRadioColor(selected.back);
        }
      else if ((menutitle=dynamic_cast<FXMenuTitle*>(w))) {
        menutitle->setTextColor(selected.fore);
        menutitle->setSelTextColor(selected.fore);
        menutitle->setSelBackColor(selected.base);
        }
      }
    else if ((menuseparator=dynamic_cast<FXMenuSeparator*>(w))) {
      menuseparator->setShadowColor(selected.shadow);
      menuseparator->setHiliteColor(selected.hilite);
      }
    else if ((scrollbar=dynamic_cast<FXScrollBar*>(w))) {
      scrollbar->setShadowColor(selected.shadow);
      scrollbar->setHiliteColor(selected.hilite);
      scrollbar->setBorderColor(selected.border);
      scrollbar->setArrowColor(selected.fore);
      }
    else if ((dragcorner=dynamic_cast<FXDragCorner*>(w))) {
      dragcorner->setShadowColor(selected.shadow);
      dragcorner->setHiliteColor(selected.hilite);
      }
    else if (dynamic_cast<FXScrollArea*>(w)) {
      if ((text=dynamic_cast<FXText*>(w))) {
        w->setBackColor(selected.back);
        text->setTextColor(selected.fore);
        text->setSelTextColor(selected.selfore);
        text->setSelBackColor(selected.selback);
        }
      else if ((list=dynamic_cast<FXList*>(w))) {
        w->setBackColor(selected.back);
        list->setTextColor(selected.fore);
        list->setSelTextColor(selected.selfore);
        list->setSelBackColor(selected.selback);
        if ((gmlist=dynamic_cast<GMList*>(w))) {
          gmlist->setRowColor(selected.altback);
          }
        }
      else if ((treelist=dynamic_cast<FXTreeList*>(w))) {
        w->setBackColor(selected.back);
        treelist->setTextColor(selected.fore);
				treelist->setLineColor(selected.shadow);
        if ((gmtreelist=dynamic_cast<GMTreeList*>(w))) {
          gmtreelist->setSelBackColor(selected.sourceback);
          gmtreelist->setSelTextColor(selected.sourcefore);
          gmtreelist->setRowColor(selected.altback);
          }
        else {
          treelist->setSelTextColor(selected.selfore);
          treelist->setSelBackColor(selected.selback);
          }
        }
      else if ((iconlist=dynamic_cast<FXIconList*>(w))) {
        w->setBackColor(selected.back);
        iconlist->setTextColor(selected.fore);
        iconlist->setSelTextColor(selected.selfore);
        iconlist->setSelBackColor(selected.selback);
        if ((gmtracklist=dynamic_cast<GMTrackList*>(w))) {
          gmtracklist->setRowColor(selected.altback);
          gmtracklist->setActiveTextColor(selected.playfore);
          gmtracklist->setActiveColor(selected.playback);
          }
        }
      else if ((foldinglist=dynamic_cast<FXFoldingList*>(w))) {
        w->setBackColor(selected.back);
        foldinglist->setTextColor(selected.fore);
        foldinglist->setSelTextColor(selected.selfore);
        foldinglist->setSelBackColor(selected.selback);
				foldinglist->setLineColor(selected.shadow);
        }
      else if ((table=dynamic_cast<FXTable*>(w))) {
        w->setBackColor(selected.back);
        table->setTextColor(selected.fore);
        table->setSelTextColor(selected.selfore);
        table->setSelBackColor(selected.selback);
        }
      }
    else if ((mdichild=dynamic_cast<FXMDIChild*>(w))) {
      mdichild->setBackColor(selected.base);
      mdichild->setBaseColor(selected.base);
      mdichild->setShadowColor(selected.shadow);
      mdichild->setHiliteColor(selected.hilite);
      mdichild->setBorderColor(selected.border);
      mdichild->setTitleColor(selected.selfore);
      mdichild->setTitleBackColor(selected.selback);
      }
    else if ((tooltip=dynamic_cast<FXToolTip*>(w))){
      tooltip->setTextColor(selected.tipfore);  
      tooltip->setBackColor(selected.tipback);
      }

    w->update(); 
    if(w->getFirst()){
      w=w->getFirst();
      continue;
      }
    while(!w->getNext() && w->getParent()){
      w=w->getParent();
      }
    w=w->getNext();
    }
  selected.save();

//GMIconTheme::instance()->load();
//GMIconTheme::instance()->updateToolbar(GMPlayerManager::instance()->getPreferences().gui_toolbar_bigicons);

  }


void GMPreferencesDialog::redraw(){
  register FXWindow *w=GMPlayerManager::instance()->getMainWindow();
  while(w){
    w->recalc();
    w->update(); 
    if(w->getFirst()){
      w=w->getFirst();
      continue;
      }
    while(!w->getNext() && w->getParent()){
      w=w->getParent();
      }
    w=w->getNext();
    }
  }


static FXString weightToString(FXuint weight){
  switch(weight){
    case FXFont::Thin      : return "thin"; break;
    case FXFont::ExtraLight: return "extralight"; break;
    case FXFont::Light     : return "light"; break;
    case FXFont::Normal    : return "normal"; break;
    case FXFont::Medium    : return "medium"; break;
    case FXFont::DemiBold  : return "demibold"; break;
    case FXFont::Bold      : return "bold"; break;
    case FXFont::ExtraBold : return "extrabold"; break;
    case FXFont::Black     : return "heavy"; break;
    default: return ""; break;
    }
  return "";
  }

static FXString slantToString(FXuint slant){
  switch(slant){
    case FXFont::Straight       : return "regular"; break;
    case FXFont::Italic         : return "italic"; break;
    case FXFont::Oblique        : return "oblique"; break;
    case FXFont::ReverseItalic  : return "reverse italic"; break;
    case FXFont::ReverseOblique : return "reverse oblique"; break;
    default : return ""; break;
    }
  return "";
  }

long GMPreferencesDialog::onCmdChangeFont(FXObject*,FXSelector,void*){
  FXFontDialog dialog(this,"Select Normal Font");
  FXFont * font = FXApp::instance()->getNormalFont();
  FXFontDesc fontdescription;
#if FOXVERSION < FXVERSION(1,7,17)
  font->getFontDesc(fontdescription);
#else
  fontdescription =  font->getFontDesc();
#endif
  strncpy(fontdescription.face,font->getActualName().text(),sizeof(fontdescription.face));
#if FOXVERSION < FXVERSION(1,7,18)
  dialog.setFontSelection(fontdescription);
  if(dialog.execute(PLACEMENT_SCREEN)){
    dialog.getFontSelection(fontdescription);
#else
  dialog.setFontDesc(fontdescription);
  if(dialog.execute(PLACEMENT_SCREEN)){
    fontdescription=dialog.getFontDesc();
#endif
    getApp()->getNormalFont()->destroy();
    getApp()->getNormalFont()->setFontDesc(fontdescription);
    getApp()->getNormalFont()->create(); 
    getApp()->reg().writeStringEntry("SETTINGS","normalfont",getApp()->getNormalFont()->getFont().text());
    GMPlayerManager::instance()->getMainWindow()->layoutToolBarButtons();  
    GMPlayerManager::instance()->getMainWindow()->getTrackView()->updateFont();
    redraw();
    FXint nw =getDefaultWidth();
    FXint nh = getDefaultHeight();
    resize(nw,nh);
    }
  return 1;
  }

long GMPreferencesDialog::onUpdFont(FXObject*sender,FXSelector,void*){
  FXFont * font = FXApp::instance()->getNormalFont();
  FXString fontname = font->getActualName().before('[').trimEnd() +", " + GMStringVal(font->getSize()/10);

  if(font->getWeight()!=0 && font->getWeight()!=FXFont::Normal){
    fontname += ", " + weightToString(font->getWeight());
    }
  if (font->getSlant()!=0 && font->getSlant()!=FXFont::Straight){
    fontname += ", " + slantToString(font->getSlant());
    }

  sender->tryHandle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETSTRINGVALUE),&fontname); 
  return 1;
  }



long GMWindow::onCmdPreferences(FXObject *,FXSelector,void*){
  GMPreferencesDialog dialog(this);
  dialog.execute();
  return 1;
  }

void GMWindow::configureStatusbar(FXbool show){
  if (show) {
    statusbar->show();
    controlstatusseparator->show();
    controldragcorner->hide();
    }
  else {
    statusbar->hide();
    controlstatusseparator->hide();
    if (!GMPlayerManager::instance()->getPreferences().gui_toolbar_docktop){
      controldragcorner->show();
      controlframe->recalc();
      }
    ///ticker->setSpeed(0);
    }
  GMPlayerManager::instance()->getPreferences().gui_show_status_bar=show;
  recalc();
  controlframe->recalc();
  }
  
  
void GMWindow::layoutToolBarButtons() {
  if (GMPlayerManager::instance()->getPreferences().gui_toolbar_showlabels) {
    FXint mw = playbutton->getDefaultWidth();
    mw = FXMAX(mw,pausebutton->getDefaultWidth());
    mw = FXMAX(mw,stopbutton->getDefaultWidth());
    mw = FXMAX(mw,prevbutton->getDefaultWidth());
    mw = FXMAX(mw,nextbutton->getDefaultWidth());

    playbutton->setLayoutHints(LAYOUT_FIX_WIDTH);
    playbutton->setWidth(mw);

    pausebutton->setLayoutHints(LAYOUT_FIX_WIDTH);
    pausebutton->setWidth(mw);

    stopbutton->setLayoutHints(LAYOUT_FIX_WIDTH);
    stopbutton->setWidth(mw);

    prevbutton->setLayoutHints(LAYOUT_FIX_WIDTH);
    prevbutton->setWidth(mw);

    nextbutton->setLayoutHints(LAYOUT_FIX_WIDTH);
    nextbutton->setWidth(mw);
    }
  else {
    playbutton->setLayoutHints(0);
    pausebutton->setLayoutHints(0);
    stopbutton->setLayoutHints(0);
    prevbutton->setLayoutHints(0);
    nextbutton->setLayoutHints(0);
    }        
  }      


void GMWindow::configureToolbar(FXbool docktop,FXbool showlabels,FXbool bigicons,FXbool init/*=false*/){

  if ((docktop != GMPlayerManager::instance()->getPreferences().gui_toolbar_docktop) || init) {
    if (docktop) {
      controlframe->setLayoutHints(LAYOUT_FILL_X|LAYOUT_SIDE_TOP);
      controlseparator->setLayoutHints(LAYOUT_FILL_X|LAYOUT_SIDE_TOP);
      controldragcorner->hide();
      }
    else {
      controlframe->setLayoutHints(LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);
      controlseparator->setLayoutHints(LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);
      if (!statusbar->shown()) controldragcorner->show();
      }
    }

  if ((bigicons != GMPlayerManager::instance()->getPreferences().gui_toolbar_bigicons) || init) {

    icontheme->loadToolbar(bigicons);

    if (bigicons) {
      playbutton->setIcon(icontheme->icon_play_big);
      pausebutton->setIcon(icontheme->icon_pause_big);
      stopbutton->setIcon(icontheme->icon_stop_big);
      prevbutton->setIcon(icontheme->icon_prev_big);
      nextbutton->setIcon(icontheme->icon_next_big);
      }
    else {
      playbutton->setIcon(icontheme->icon_play);
      pausebutton->setIcon(icontheme->icon_pause);
      stopbutton->setIcon(icontheme->icon_stop);
      prevbutton->setIcon(icontheme->icon_prev);
      nextbutton->setIcon(icontheme->icon_next);
      }
    update_volume_display(GMPlayerManager::instance()->volume());
    }

  if (showlabels != GMPlayerManager::instance()->getPreferences().gui_toolbar_showlabels || init) {
    if (showlabels) {
      playbutton->setText("Play");
      pausebutton->setText("Pause");
      stopbutton->setText("Stop");
      prevbutton->setText("Previous");
      nextbutton->setText("Next");
      }
    else {
      playbutton->setText(FXString::null);
      pausebutton->setText(FXString::null);
      stopbutton->setText(FXString::null);
      prevbutton->setText(FXString::null);
      nextbutton->setText(FXString::null);
      }
    GMPlayerManager::instance()->getPreferences().gui_toolbar_showlabels=showlabels;
    layoutToolBarButtons();  
    }
  GMPlayerManager::instance()->getPreferences().gui_toolbar_docktop=docktop;
  GMPlayerManager::instance()->getPreferences().gui_toolbar_bigicons=bigicons;
  }
















long GMWindow::onCmdImportFiles(FXObject *,FXSelector,void*){
  static const FXchar patterns[]="All Music (*.mp3,*.ogg,*.flac,*.mpc"
#ifndef DISABLE_MP4_SUPPORT
",*.aac,*.m4a,*.m4p,*.mp4"
#endif
#ifndef DISABLE_ASF_SUPPORT
",*.wma,*,asf"
#endif
")\nFree Lossless Audio Codec (*.flac)\n"
"MPEG-1 Audio Layer 3 (*.mp3)\n"
#ifndef DISABLE_MP4_SUPPORT
"MPEG-4 Part 14 (*.mp4,*.m4a,*.m4p,*.aac)\n"
#endif
"Musepack (*.mpc)\n"
"Ogg Vorbis (*.ogg)\n"
#ifndef DISABLE_ASF_SUPPORT
"Window Media (*.wma,*,asf)\n"
#endif
"All Files (*.*)";

  FXString searchdir = getApp()->reg().readStringEntry("directories","last-import-files",FXSystem::getHomeDirectory().text());
  FXFileDialog dialog(this,"Import Music: Select File(s) to import...");
  dialog.setDirectory(searchdir);
  dialog.setSelectMode(SELECTFILE_MULTIPLE);
  dialog.setPatternList(patterns);
  dialog.setCurrentPattern(0);
  dialog.setMatchMode(FILEMATCH_CASEFOLD);

#if FOXVERSION >= FXVERSION(1,7,11)
  FXSettings empty;
  FXFileDict filedict(getApp(),&empty);
  icontheme->loadFileAssociations(filedict);
  dialog.setAssociations(&filedict,false);
#endif

  if (dialog.execute()){
    FXStringList list;
    FXString * files = dialog.getFilenames();
    if (!files) return 1;
    for (FXint i=0;files[i]!=FXString::null;i++) {
      list.append(files[i]);
      }
    delete [] files;
    GMPlayerManager::instance()->stop();
    searchdir = dialog.getDirectory();
    GMSearchDialog searchdialog(this,list,-1,GMPlayerManager::instance()->getPreferences(),font_thick);
    searchdialog.execute();
    getApp()->reg().writeStringEntry("directories","last-import-files",searchdir.text());
    if (sourceview->getSource()==GMPlayerManager::instance()->getSource(0))
      trackview->refresh();
    }

#if FOXVERSION >= FXVERSION(1,7,11)
  icontheme->freeFileAssociations();
#endif
  return 1;
  }


long GMWindow::onCmdImportDirs(FXObject *,FXSelector,void*){
  FXString searchdir = getApp()->reg().readStringEntry("directories","last-import-dirs",FXSystem::getHomeDirectory().text());
  FXDirDialog dialog(this,"Import Music: Select Directory to import music files from...");
  dialog.setDirectory(searchdir);

#if FOXVERSION >= FXVERSION(1,7,11)
  FXSettings    empty;
  FXFileDict filedict(getApp(),&empty);
  icontheme->loadDirAssociations(filedict);
  dialog.setAssociations(&filedict,false);
#endif



  if (dialog.execute()){
    FXStringList list;
    searchdir = dialog.getDirectory();
    list.append(searchdir);
    GMPlayerManager::instance()->stop();
    GMSearchDialog searchdialog(this,list,-1,GMPlayerManager::instance()->getPreferences(),font_thick);
    searchdialog.execute();
    getApp()->reg().writeStringEntry("directories","last-import-dirs",searchdir.text());
    if (sourceview->getSource()==GMPlayerManager::instance()->getSource(0))
      trackview->refresh();
    }

#if FOXVERSION >= FXVERSION(1,7,11)
  icontheme->freeDirAssociations();
#endif

  return 1;
  }










long GMWindow::onCmdPlay(FXObject*,FXSelector,void*){
  if (GMPlayerManager::instance()->can_unpause())
    GMPlayerManager::instance()->unpause();
  else
    GMPlayerManager::instance()->play();
  return 1;
  }

long GMWindow::onUpdPlay(FXObject*sender,FXSelector,void*){
  if ( GMPlayerManager::instance()->can_play() || GMPlayerManager::instance()->can_unpause())
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
  return 1;
  }


long GMWindow::onCmdPause(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->pause();
  return 1;
  }

long GMWindow::onUpdPause(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->can_pause())
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
  return 1;
  }

long GMWindow::onCmdStop(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->stop();
  return 1;
  }

long GMWindow::onUpdStop(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->can_stop())
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
  return 1;
  }

long GMWindow::onCmdNext(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->next();
  return 1;
  }

long GMWindow::onUpdNext(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->can_next())
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
  return 1;
  }

long GMWindow::onCmdPrev(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->prev();
  return 1;
  }

long GMWindow::onUpdPrev(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->can_prev())
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
  return 1;
  }


long GMWindow::onCmdTimeSlider(FXObject*,FXSelector,void*ptr){
  FXint pos=(FXint)(FXival)ptr;
  GMPlayerManager::instance()->seek(pos);
  return 1;
  }

long GMWindow::onCmdVolume(FXObject*,FXSelector,void*ptr){
  FXint level = (FXint)(FXival)ptr;
  GMPlayerManager::instance()->volume(level);
  update_volume_display(level);
  return 1;
  }

long GMWindow::onCmdVolumeButton(FXObject*,FXSelector sel,void*ptr){
  volumeslider->handle(this,FXSEL(FXSELTYPE(sel),0),ptr);
  return 1;
  }

long GMWindow::onUpdVolumeButton(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->getPlayer()->opened())
    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
  return 1;
  }

long GMWindow::onCmdRepeatAB(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->getPlayer()->setRepeatAB();
  return 1;
  }

long GMWindow::onUpdRepeatAB(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->getPlayer()->playing() && GMPlayerManager::instance()->getPlayer()->seekable()) {
    const FXString state_off = "Repeat A-B";
    const FXString state_a = "Repeat A";
    const FXString state_b = "Repeat A-B";
    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
    switch(GMPlayerManager::instance()->getPlayer()->getRepeatAB()){
      case REPEAT_AB_OFF: sender->handle(this,FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
                          sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&state_off);
                          break;
      case REPEAT_AB_A  : sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
                          sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&state_a);
                          break;
      case REPEAT_AB_B  : sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
                          sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&state_b);
                          break;
      }
    }
  else {
    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
    }
  return 1;
  }



long GMWindow::onCmdRepeatOff(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->getPreferences().play_repeat=REPEAT_OFF;
  return 1;
  }

long GMWindow::onUpdRepeatOff(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->getPreferences().play_repeat==REPEAT_OFF)
    sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
  return 1;
  }

long GMWindow::onCmdRepeat(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->getPreferences().play_repeat=REPEAT_TRACK;
  return 1;
  }

long GMWindow::onUpdRepeat(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->getPreferences().play_repeat==REPEAT_TRACK)
    sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
  return 1;
  }


long GMWindow::onCmdRepeatAll(FXObject*,FXSelector,void*){
  GMPlayerManager::instance()->getPreferences().play_repeat=REPEAT_ALL;
  return 1;
  }

long GMWindow::onUpdRepeatAll(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->getPreferences().play_repeat==REPEAT_ALL)
    sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
  return 1;
  }


long GMWindow::onCmdShuffle(FXObject*,FXSelector,void*ptr){
  GMPlayerManager::instance()->getPreferences().play_shuffle = (FXbool)(FXival)(ptr);
  return 1;
  }

long GMWindow::onUpdShuffle(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->getPreferences().play_shuffle)
    sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
  return 1;
  }


long GMWindow::onCmdHomePage(FXObject*,FXSelector,void*){
  if (!launchBrowser("http://code.google.com/p/gogglesmm")){
    FXMessageBox::error(this,MBOX_OK,"Unable to launch webbrowser","Goggles Music Manager was unable to launch a webbrowser.\nPlease visit http://code.google.com/p/gogglesmm for the official homepage.");
    }
  return 1;
  }

long GMWindow::onCmdReportIssue(FXObject*,FXSelector,void*){
  if (!launchBrowser("http://code.google.com/p/gogglesmm/issues/list")){
    FXMessageBox::error(this,MBOX_OK,"Unable to launch webbrowser","Goggles Music Manager was unable to launch a webbrowser.\nPlease visit http://code.google.com/p/gogglesmm/issues/list to report an issue.");
    }
  return 1;
  }

long GMWindow::onCmdJoinLastFM(FXObject*,FXSelector,void*){
  if (!launchBrowser("https://www.last.fm/join/")){
    FXMessageBox::error(this,MBOX_OK,"Unable to launch webbrowser","Goggles Music Manager was unable to launch a webbrowser.\nPlease visit https://www.last.fm/join/");
    }
  return 1;
  }

long GMWindow::onCmdJoinGMMLastFM(FXObject*,FXSelector,void*){
  if (!launchBrowser("http://www.last.fm/group/Goggles+Music+Manager")){
    FXMessageBox::error(this,MBOX_OK,"Unable to launch webbrowser","Goggles Music Manager was unable to launch a webbrowser.\nPlease visit http://www.last.fm/group/Goggles+Music+Manager");
    }
  return 1;
  }


long GMWindow::onCmdOpen(FXObject*,FXSelector,void*){
  FXString mrl;
  if (FXInputDialog::getString(mrl,this,"Open MRL","Please specify a mrl to play",NULL)){
    GMPlayerManager::instance()->play(mrl);
    }
  return 1;
  }


long GMWindow::onCmdSleepTimer(FXObject*,FXSelector,void*ptr){
  if (((FXint)(FXival)ptr)==1) {
    FXDialogBox dialog(this,"Sleep Timer",DECOR_TITLE|DECOR_BORDER,0,0,0,0,0,0,0,0,0,0);
    create_dialog_header(&dialog,tr("Setup sleep timer"),tr("Stop playback within a certain time"),NULL);
    FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X,0,0,0,0);
    new FXButton(closebox,tr(" &Start Timer "),NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
    new FXButton(closebox,tr(" &Cancel "),NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
    new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

    FXHorizontalFrame * main = new FXHorizontalFrame(&dialog,LAYOUT_FILL_X,0,0,0,0,40,40,10,10);
    new FXLabel(main,tr("Sleep in"),NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    FXSpinner* hours = new FXSpinner(main,2,NULL,0,FRAME_SUNKEN|FRAME_THICK);
    new FXLabel(main,"hours and",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    FXSpinner* minutes = new FXSpinner(main,2,NULL,0,FRAME_SUNKEN|FRAME_THICK);
    new FXLabel(main,"minutes.",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    minutes->setRange(0,59);
    hours->setRange(0,24);

    minutes->setValue(FXCLAMP(0,getApp()->reg().readIntEntry("player","sleeptimer-minutes",0),59));
    hours->setValue(FXCLAMP(0,getApp()->reg().readIntEntry("player","sleeptimer-hours",2),24));

    if (dialog.execute()) {
      GMPlayerManager::instance()->setSleepTimer(TIME_HOUR(hours->getValue())+TIME_MIN(minutes->getValue()));
      getApp()->reg().writeIntEntry("player","sleeptimer-minutes",minutes->getValue());
      getApp()->reg().writeIntEntry("player","sleeptimer-hours",hours->getValue());
      }
    }
  else {
    GMPlayerManager::instance()->setSleepTimer(0);
    }
  return 1;
  }

long GMWindow::onUpdSleepTimer(FXObject*sender,FXSelector,void*){
  if (GMPlayerManager::instance()->hasSleepTimer())
    sender->handle(this,FXSEL(SEL_COMMAND,ID_CHECK),NULL);
  else
    sender->handle(this,FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
  return 1;
  }


extern const char *albumartfiles[];

void GMWindow::loadCover(const FXString & filename) {
  FXImage * cover=NULL;
  FXIconSource src(getApp());
  FXString dirname;

  if (filename==coverfile) {
    return;
    }

  dirname=filename;
  cover = GMTag::albumart(filename,128);
  if (cover) goto found_cover;

  dirname = FXPath::directory(filename);
  if (dirname==coverfile) {
    return;
    }

  if (!dirname.empty()) {
    FXString * files=NULL;
    FXString * names=NULL;
    FXint i,c;

    FXint nfound = FXDir::listFiles(files,dirname,"*.(png,jpg,jpeg,bmp,gif)",FXDir::NoDirs|FXDir::NoParent|FXDir::CaseFold|FXDir::HiddenFiles);
    if (nfound>1) { // find the best one, based on albumartfiles preference...

      names = new FXString[nfound];
      for (i=0;i<nfound;i++) 
        names[i]=FXPath::title(files[i]);

      for (c=0;albumartfiles[c]!=NULL;c++) {
        for (FXint i=0;i<nfound;i++){
          if (comparecase(names[i],albumartfiles[c])==0) {
            cover = src.loadScaledImageFile(dirname+PATHSEPSTRING+files[i],128,1);
            if (cover) {
              delete [] files;
              delete [] names;
              goto found_cover;
              }
            }
          }
        }      
      delete [] names;
      }
    if (nfound>=1) {
      cover = src.loadScaledImageFile(dirname+PATHSEPSTRING+files[0],128,1);
      }
    delete [] files;
    }

found_cover:  

  if (coverframe->getImage()){
    delete coverframe->getImage();
    }

  if (GMPlayerManager::instance()->getRemote())
    GMPlayerManager::instance()->getRemote()->updateCover(NULL);

  if (cover_small)  {
    delete cover_small;
    cover_small=NULL;
    }

  coverframe->setImage(cover);

  if (cover) {
    coverfile = dirname;

    FXColor * imagedata = cover->getData();
    FXuint opts=cover->getOptions();
    opts|=IMAGE_KEEP;
    cover->setOptions(opts);
    cover->create();

    cover_small = new FXImage(getApp(),imagedata,IMAGE_SHMI|IMAGE_SHMP,cover->getWidth(),cover->getHeight());
    if (cover_small->getWidth()>64 || cover_small->getHeight()>64) {
      if (cover_small->getWidth()>cover_small->getHeight())
        cover_small->scale(64,(64*cover_small->getHeight())/cover_small->getWidth(),1);
      else
        cover_small->scale((64*cover_small->getWidth())/cover_small->getHeight(),64,1);
      }

#ifdef HAVE_DBUS
  if (GMPlayerManager::instance()->getPreferences().dbus_notify_daemon) {
    FXFileStream file;
    if (file.open(FXSystem::getHomeDirectory() + PATHSEPSTRING + ".goggles" + PATHSEPSTRING + "cover.jpg",FXStreamSave)){
      fxsaveJPG(file,cover_small->getData(),cover_small->getWidth(),cover_small->getHeight(),75);
      file.close();
      }
    }
#endif

    cover_small->create();
    cover->release();

    if (GMPlayerManager::instance()->getRemote())
      GMPlayerManager::instance()->getRemote()->updateCover(cover_small);

    coverframe->show();
    coverframe->recalc();
    }
  else {
    coverfile = FXString::null;
    coverframe->hide();
    coverframe->recalc();
    }
  }



void GMWindow::create_dialog_header(FXDialogBox * dialog,const FXString & title,const FXString & subtitle,FXIcon * icon) {
  FXHorizontalFrame * header = new FXHorizontalFrame(dialog,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  header->setBackColor(getApp()->getBackColor());
  FXLabel * label = new FXLabel(header,FXString::null,icon,LABEL_NORMAL|LAYOUT_CENTER_Y,0,0,0,0,10,0,0,0);
  label->setBackColor(getApp()->getBackColor());
  FXVerticalFrame * frame = new FXVerticalFrame(header,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  label = new FXLabel(frame,title,NULL,LAYOUT_FILL_X|JUSTIFY_LEFT|TEXT_AFTER_ICON,0,0,0,0,10,10,10,0);
  label->setBackColor(getApp()->getBackColor());
  label->setFont(font_thick);
  label = new FXLabel(frame,subtitle,NULL,LAYOUT_FILL_X|JUSTIFY_LEFT|TEXT_AFTER_ICON,0,0,0,0,10+30,10,0,10);
  label->setBackColor(getApp()->getBackColor());
  new FXSeparator(dialog,LAYOUT_FILL_X|SEPARATOR_GROOVE|LAYOUT_SIDE_TOP);
  }
