/*******************************************************************************
*                    Tray Icon for Goggles Music Manager                       *
********************************************************************************
*           Copyright (C) 2008 by Olivier Duclos. All Rights Reserved          *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/

/* This tray icon is a contribution
 * by Olivier Duclos <olivier.duclos gmail.com>
 */

#include <gtk/gtk.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>

#define GOGGLESMM_DBUS_NAME "org.fifthplanet.gogglesmm"
#define GOGGLESMM_DBUS_PATH "/org/fifthplanet/gogglesmm"
#define GOGGLESMM_DBUS_INTERFACE "org.fifthplanet.gogglesmm"

static GtkWidget *my_menu = NULL;
static GtkStatusIcon *status_icon = NULL;
static DBusGProxy *proxy = NULL;
static GtkWidget *play_menu_item = NULL;
static GtkWidget *pause_menu_item = NULL;
static GtkWidget *stop_menu_item = NULL;
static GtkWidget *next_menu_item = NULL;
static GtkWidget *prev_menu_item = NULL;

enum {
		CAN_PLAY  		= 0x1,
		CAN_PAUSE 		= 0x2,
		CAN_STOP			= 0x4,
		CAN_NEXT  		= 0x8,
		CAN_PREV			= 0x10,
		};


static void
play(GtkWidget *widget, gpointer data)
{
	dbus_g_proxy_call_no_reply(proxy, "play",G_TYPE_INVALID, G_TYPE_INVALID);
}

static void
pause(GtkWidget *widget, gpointer data)
{
	dbus_g_proxy_call_no_reply(proxy, "pause",G_TYPE_INVALID, G_TYPE_INVALID);
}

static void
stop(GtkWidget *widget, gpointer data)
{
	dbus_g_proxy_call_no_reply(proxy, "stop",G_TYPE_INVALID, G_TYPE_INVALID);
}

static void
previous(GtkWidget *widget, gpointer data)
{
	dbus_g_proxy_call_no_reply(proxy, "prev",G_TYPE_INVALID, G_TYPE_INVALID);
}

static void
next(GtkWidget *widget, gpointer data)
{
	dbus_g_proxy_call_no_reply(proxy, "next",G_TYPE_INVALID, G_TYPE_INVALID);
}

static void
destroy(GtkWidget *widget, gpointer data)
{
	dbus_g_proxy_call_no_reply(proxy, "exit",G_TYPE_INVALID, G_TYPE_INVALID);
	gtk_main_quit ();
}

static void
destroy_signal(GtkWidget *widget, gpointer data)
{
	//g_debug("'destroy' signal triggered");
	gtk_main_quit ();
}


static void
showhide (GtkStatusIcon* status_icon, gpointer user_data)
{
	dbus_g_proxy_call_no_reply(proxy, "toggleshown",G_TYPE_INVALID, G_TYPE_INVALID);
}

static void
popup(GtkStatusIcon *status_icon,
	guint button,
	guint activate_time,
	gpointer user_data)
{
	guint actions=0;
	GError *error = NULL;


	if (!my_menu)
	{
		GtkWidget * item=NULL;
		my_menu = gtk_menu_new();

		play_menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_PLAY, NULL);
		gtk_menu_append(my_menu, play_menu_item);
		g_signal_connect(G_OBJECT(play_menu_item), "activate",
				G_CALLBACK(play), NULL);

		pause_menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_PAUSE, NULL);
		gtk_menu_append(my_menu, pause_menu_item);
		g_signal_connect (G_OBJECT(pause_menu_item), "activate",
				G_CALLBACK(pause), NULL);

		stop_menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_STOP, NULL);
		gtk_menu_append(my_menu, stop_menu_item);
		g_signal_connect (G_OBJECT(stop_menu_item), "activate",
				G_CALLBACK(stop), NULL);

		item = gtk_separator_menu_item_new();
		gtk_menu_append(my_menu, item);

		prev_menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_PREVIOUS, NULL);
		gtk_menu_append(my_menu, prev_menu_item);
		g_signal_connect (G_OBJECT(prev_menu_item), "activate",
				G_CALLBACK(previous), NULL);

		next_menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_NEXT, NULL);
		gtk_menu_append(my_menu, next_menu_item);
		g_signal_connect (G_OBJECT(next_menu_item), "activate",
				G_CALLBACK(next), NULL);

		item = gtk_separator_menu_item_new();
		gtk_menu_append(my_menu, item);

		item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
		gtk_menu_append(my_menu, item);
		g_signal_connect (G_OBJECT(item), "activate",
				G_CALLBACK(destroy), NULL);
	}

	
	if (dbus_g_proxy_call(proxy,"getactions",&error,G_TYPE_INVALID,G_TYPE_UINT,&actions,G_TYPE_INVALID)){
	  gtk_widget_set_sensitive(GTK_WIDGET(play_menu_item),(actions&CAN_PLAY));
	  gtk_widget_set_sensitive(GTK_WIDGET(pause_menu_item),(actions&CAN_PAUSE));
	  gtk_widget_set_sensitive(GTK_WIDGET(stop_menu_item),(actions&CAN_STOP));
	  gtk_widget_set_sensitive(GTK_WIDGET(next_menu_item),(actions&CAN_NEXT));
	  gtk_widget_set_sensitive(GTK_WIDGET(prev_menu_item),(actions&CAN_PREV));
		}
	else {
	  gtk_widget_set_sensitive(GTK_WIDGET(play_menu_item),TRUE);
	  gtk_widget_set_sensitive(GTK_WIDGET(pause_menu_item),TRUE);
	  gtk_widget_set_sensitive(GTK_WIDGET(stop_menu_item),TRUE);
	  gtk_widget_set_sensitive(GTK_WIDGET(next_menu_item),TRUE);
	  gtk_widget_set_sensitive(GTK_WIDGET(prev_menu_item),TRUE);	
		g_printerr ("Error: %s\n", error->message);
		g_error_free (error);
		}	
		
		
	gtk_widget_show_all(my_menu);

	gtk_menu_popup(GTK_MENU(my_menu),
			NULL,
			NULL,
			gtk_status_icon_position_menu,
			status_icon,
			button,
			activate_time);
}

int main( int argc, char* argv[] )
{
	DBusGConnection *conn;
	GError *err = NULL;

	gtk_init(&argc, &argv);

	status_icon = gtk_status_icon_new_from_icon_name("gmm-tray");
	gtk_status_icon_set_visible(status_icon, TRUE);
	gtk_status_icon_set_tooltip(status_icon, "Goggles Music Manager");

	/* Connect signals */
	g_signal_connect (G_OBJECT (status_icon), "popup-menu",
			  G_CALLBACK (popup), NULL);

	g_signal_connect (G_OBJECT (status_icon), "activate",
			  G_CALLBACK (showhide), NULL);

	/* Setup the DBUS proxy */
	conn = dbus_g_bus_get (DBUS_BUS_SESSION, &err);
	if (!conn) {
		g_printerr ("Error: %s\n", err->message);
		g_error_free (err);
		return 0;
	}

	proxy = dbus_g_proxy_new_for_name_owner (conn,
                                  GOGGLESMM_DBUS_NAME,
                                  GOGGLESMM_DBUS_PATH,
                                  GOGGLESMM_DBUS_INTERFACE,NULL);

	if (proxy==NULL) {
		g_printerr ("Goggles Music Manager is not running.\n");
		return 0;
	}


	/* Listening to gmm's signals */
	g_signal_connect (G_OBJECT (proxy), "destroy",G_CALLBACK (destroy_signal), NULL);

	/// Music Manager quits
	dbus_g_proxy_add_signal(proxy,"quit",G_TYPE_INVALID,G_TYPE_INVALID);
	dbus_g_proxy_connect_signal(proxy,"quit", G_CALLBACK (destroy_signal),NULL,NULL);

	/// Tray Should Quit
	dbus_g_proxy_add_signal(proxy,"trayexit",G_TYPE_INVALID,G_TYPE_INVALID);
	dbus_g_proxy_connect_signal(proxy,"trayexit", G_CALLBACK (destroy_signal),NULL,NULL);

	gtk_main();

	return 0;
}
